### A Pluto.jl notebook ###
# v0.19.14

using Markdown
using InteractiveUtils

# ╔═╡ 369a24c8-6f09-11eb-16a9-9f2c6c94a77f
begin
    using Pkg, LinearAlgebra, PlutoUI, TestImages, Plots 

    hint(text) = Markdown.MD(Markdown.Admonition("hint", "Hint", [text]))
    note(text) = Markdown.MD(Markdown.Admonition("note", "Note", [text]))
    example(text) = Markdown.MD(Markdown.Admonition("note", "Example", [text]))

	
    PlutoUI.TableOfContents()
end

# ╔═╡ 349d7534-6212-11eb-2bc5-db5be39b6bb6
md"""
# Scientific Programming - Introduction to Julia Programming

[Institute for Biomedical Imaging](https://www.tuhh.de/ibi/home.html), Hamburg University of Technology

* 👨‍🏫 Prof. Dr.-Ing. Tobias Knopp
* 👨‍🏫 Dr. rer. nat. Martin Möddel
"""

# ╔═╡ fcb548d6-672a-11eb-3167-e7da4dfc9126
md"""
## Getting Started with the REPL

Julia code can be run in various ways:
* The easiest way is to start the Julia REPL. REPL stands for Read Eval Print Loop and is what is often known as a terminal or console. The REPL is characterized by being an interactive environoment. One can basically inspect the code while running it. When you start Julia from the command line you will get a promt like this
```julia
julia> 1 + 1
2
```
What happens is that the *expression* `1+1` is *read*. Then it is *evaluated*. And finally the result is *printed*. All this is repeated in an infinite *loop*. So its a Read Eval Print Loop.

* It is also possible to execute a Julia script that has the extension .jl by running 

```bash
  > julia myscript.jl
```
"""

# ╔═╡ 6ca05a0a-915b-11eb-07d2-a7277ce215d6
note(md"One common question being asked when learning a new programming language is: What is a good IDE (Integrated Development Environment) for the language? 
	
We actually encourage that you start with a simple text editor and the REPL and learn how to master programming in this way. As editors we suggest
	
	* vim
	* vscode
	* gedit
	
just to name a few. `vim` is a tool, which you should get used to anyway since you will have situations where you are on a server and want to change some code without a graphical frontend.
	
Note that `vscode` is actually a full featured Julia IDE with the appropriate plugin being installed. So if you want to give that a go feel free to play around with its features.
	")

# ╔═╡ edd72f7e-9159-11eb-220a-77decc54c26b
md" ## Notebooks

In this course we use *Pluto.jl* for both the lecture and the exercise. *Pluto* is a so-called *notebook* system which is helpful in an interactive setting. Pluto is similar to Jupyter or Mathematica but has some advantages compared to Jupyter. In particular the current document is a plain Julia file that you can also execute in the Julia REPL or open in a plain text editor like vim or vscode. Here are some Pluto features:

* One can mix code, text, and images freely.
* Notebooks are an implementation of [literate programming](https://en.wikipedia.org/wiki/Literate_programming) proposed by Donald Knuth. Notebooks are actually a little bit more, the term *literate computing* is often used for notebook systems.
* In notebook systems one has *cells* that are executed in order.
* Notebooks are also handy to capture the *plotting results* being generated during a calculation.
* Pluto also has a nice help function (see live docs in the lower right)

But we should note that despite these many advantages, notebook systems also have disadvantages:

* They do not scale well when modularizing code.
* Video explaining several disadvantage of notebook systems on [YouTube](https://www.youtube.com/watch?v=7jiPeIFXb6U).

"

# ╔═╡ 44cd3c42-915a-11eb-060f-f705832f2ec2
note(md"We encourage you to use the REPL early although the first exercises are all done using Pluto notebooks. The reason is that this help us putting the tasks into small chunks that you can solve step by step. In a real project you will develop the code in a package and only use notebooks as a frontend.")

# ╔═╡ 679ce7ce-9176-11eb-0522-8fcbd56eb606
note(md"Within this notebook you will sometime see expressions like 
	
```julia
with_terminal() do
	...
end
``` 
or
```julia
begin
	...
end
```
or
```julia
let ...
	...
end
```
You can simply ignore these enclosing code blocks. Both are necessary in *Pluto* to display multiple expressions within a single cell. From time to time Julia code is not entered as real code but just as text as is done within this note. We do this when we don't want the code to run, for example because the above code is not complete. 
")

# ╔═╡ 581a8266-672f-11eb-0945-ff78763e7ffe
md"""
## Assignments

Values can be assigned to variables using the assignment operator
"""

# ╔═╡ 6d646d66-672e-11eb-3380-45208037b0fd
a = 30

# ╔═╡ 9849f642-672f-11eb-2ccf-db35a28f9623
md"""Coming from a language with static typing one immediately observes
that the type of the variable does not need to be declared. This is usual for
a language with a *dynamic type system*. But of course `a` still has a concrete type. Lets inspect that:
"""

# ╔═╡ 1df4f1f4-6730-11eb-3577-cfce50db2306
typeof(a)

# ╔═╡ 238263a4-6730-11eb-3f01-c727f2a2653f
md"""
This makes sense since the literal `30` apparently is an integer. Julia's default integer type has 64 bit. Here, are some elementary types:
"""

# ╔═╡ a26d2fb6-6730-11eb-0335-5903a9c7e1e3
typeof(0.4)

# ╔═╡ b2b38c2e-6730-11eb-16cb-dbf9b0bc77c6
typeof("Test")

# ╔═╡ c74fbb44-6730-11eb-2d65-d399710e4731
typeof(true)

# ╔═╡ cecc9b94-6730-11eb-0fbe-af224c6d9333
typeof(pi)

# ╔═╡ dae44ee0-6730-11eb-3d9e-070805ec03f8
md"""
That is interesting, why isn't the type of the constant `pi` a `Float64`? Maybe because pi does not fit into a 64-bit floating point number. Lets convert it
"""

# ╔═╡ 15d5224c-6731-11eb-3cac-7965f2219d70
convert(Float64, pi)

# ╔═╡ 4aa92996-6731-11eb-2f37-cf4c344cfe36
md"""
This brings us to an interesting aspect. What if we want a variable `b` to be of a specific type. Can this somehow be enforced? The answer is yes and the syntax is as follows
"""

# ╔═╡ 2c28625a-6731-11eb-2a7b-fb9a8cc66c4c
b::Float64 = pi

# ╔═╡ 41cbcdae-6731-11eb-0744-e37ed9e38bd6
typeof(b)

# ╔═╡ 8b1cf122-6731-11eb-3cc3-313aa4eeeaad
md"""
We call this explicit typing. You can also see in the example that the conversion from the type `Irrational{:π}` to the type `Float64` is done implicitly. This is because the function `convert(::Type{Float64}, ::Irrational{:π})` exists. Implicit conversion is very handy. You are certainly used to it from other programming languages
"""

# ╔═╡ 99738086-9140-11eb-3976-871ba51b4001
md"## Printing

The Julia REPL prints the value of an entered expression by default. One can omit this by putting a `;` at the end of an expression.
"

# ╔═╡ 983ad5ba-9141-11eb-189f-49719162960f
note(md"Unlike Matlab Julia does not require semicolons in scripts that are included to suppress printing a value. When you include a script what is printed is the last expression being evaluated.")

# ╔═╡ e99b310c-9141-11eb-00dd-2b12f8ff306d
md"In Julia code in files one thus needs functions to print variables and expressions. There are various possibilities for that:"

# ╔═╡ aac7e61a-9140-11eb-3ef9-479016cc327f
with_terminal() do
	y = 3
	
	println(y) # if you omit the ln, there will be no line break
	@show y # this macro is handy since it can operate on expressions
	@info y # this goes in direction of the logging infrastructure
	@warn "What is going on?"
	
end

# ╔═╡ 6a905992-9142-11eb-0661-e934fb259bcd
md"""Of course we can also read user input:

```julia
julia> a = readline();
Hallo

julia> @show a
a = "Hallo"
"Hallo"
```
"""

# ╔═╡ 68d9602e-6730-11eb-3749-c326974bbc32
md"""
## Functions
One of the core concepts in all successful programming languages is the possibility of grouping functionality in smaller, reusable code units called functions. Functions typically have arguments, a function body, and return values.

Lets define a function that calculates square numbers
"""

# ╔═╡ ef5b41b0-90cf-11eb-10d6-c77a24888854
function square(a)
  b = a*a
  return b
end

# ╔═╡ 0ec6d94e-90d0-11eb-280f-c72dae3ba87a
square(4)

# ╔═╡ 149d66a8-90d0-11eb-0d06-4d0136e49046
md"Seems to work. Here, `a` is the function argument, `b` is the return value and everything between the first and the last line is the function body. Julia also has a short notation:

```julia
square(a) = a*a
```
which is often handy when defining small functions.
"

# ╔═╡ ef167287-8a5c-42b5-a201-53a01f49d771
note(md"Functions in Julia are as for most procedural languages not the same as mathematical functions since they can have side effects. Julia has the possibility of marking functions as pure, which allows the compiler to apply additional optimization that are not allowed for unpure functions")

# ╔═╡ 55589bec-90d1-11eb-2f86-45ecfcd979aa
md"### Higher Order Functions
In Julia functions are regular objects of type `Function`. Thus they can be stored or passed around freely, i.e. functions in Julia are [first-class objects](https://en.wikipedia.org/wiki/First-class_citizen). This allows to implement higher order functions in Julia:
"

# ╔═╡ 8520088a-90d1-11eb-1315-d18ad7cb7272
function foo(f::Function, arg)
	return f(arg)
end

# ╔═╡ f12348cc-90d2-11eb-3a36-474cb43c61e5
foo(square, 4)

# ╔═╡ 0a6f00da-90d3-11eb-0809-17e905c457bb
md"Such higher order functions are for instance relevant when wanting to apply scalar function to a collection of values:"

# ╔═╡ 866f0c20-90d3-11eb-3bf4-c1d1b8fcf7d0
map(square, [1, 2, 3])

# ╔═╡ 92158d40-90d0-11eb-1a22-7dcb06bad9c0
md"### Anonymous Functions
Functions do not necessary need to have a name (like `square` in the previous example). We call a function without a name an *anonymous* function. It can be defined like this:
"

# ╔═╡ 7f150096-90d4-11eb-12ff-b30b4678f278
map(a -> a*a, [1, 2, 3])

# ╔═╡ b79a024a-90d4-11eb-3dd5-e9a39c72094b
md"(more information: [Julia Documentation]
(https://docs.julialang.org/en/v1/manual/functions/#man-anonymous-functions))"

# ╔═╡ 9fbf1526-6f09-11eb-1fa1-bd7168b987af
md"### Optional Arguments

In many cases the arguments of a function can be divided into *mandatory* and *optional* arguments. This can be done like this:
"

# ╔═╡ 5ec086c6-90d9-11eb-0606-6d7c3252d55b
mylog(a,b=ℯ) = log(a,b)

# ╔═╡ 7ad77c7e-90da-11eb-2082-2ff8ffb611d0
md" Here we defined a function `mylog` that by default uses the exponent `e` for the calculation of the logarithm. We thus can call it like this:"

# ╔═╡ b02fe60e-90da-11eb-2808-7f013ef47d5e
mylog(3.0)

# ╔═╡ dd18ebc0-90da-11eb-3969-63f78fffa36a
md"I.e. the second argument has been omitted. We note that the second argument to `log` actually already is a positional argument."

# ╔═╡ 2c393392-673d-4f8d-a658-0572817eb795
note(md"Positional arguments are usually used when a small number of arguments is optional and when there is a clear hierachy, which argument is *more* optional.")

# ╔═╡ 851a5dd6-2580-412a-8ba6-ba724c56d265
md"(more information: [Julia Documentation]
(https://docs.julialang.org/en/v1/manual/functions/#Optional-Arguments))"

# ╔═╡ 99d34166-6f09-11eb-0c40-855d2ea50483
md"### Keyword Arguments

While positional arguments are very useful, they have limitations. Lets consider the function 
```julia
function foo(a=1, b=2, c=3)
...
end
```
One can easily call this function as 
```julia
foo()
foo(1)
foo(1,2)
foo(1,2,3)
```
But what is if we just want to pass `c` to `foo` and use the default values for `a` and `b`? This is not possible.
"

# ╔═╡ 3ab650b8-66c1-4b1a-8e02-ea4f122b1f90
md"Keyword arguments can solve this problem and are nowadays a crucial element in many Julia APIs. The idea is to give the argument a name (i.e. key) and then it is possible to change the order of arguments.

As an example we consider the a function
```julia
plot(x,y)
```
that is supposed to plot a graph with coordinates `x,y`. Such a function can have
several parameters such as
* the line width
* the line style
* the color of the graph
"

# ╔═╡ a5a4b622-8b85-4d3b-b469-79d125d5b0b2
md"
In Julia we can define `plot` like this:
"

# ╔═╡ 334b9391-74e8-4778-bd71-eebc54d90fde
function myplot(x,y; width=1, style="solid", color="green")
	@info width, style, color
	# here do the actual plotting
end

# ╔═╡ b58c1f71-11b7-4dc5-8ebb-69f9e983f7a3
md"Let us see how we can call this function."

# ╔═╡ 7b5abd7f-68de-43fc-bd6f-8ecc8a62a8c5
with_terminal() do
	x = collect(1:10)
	y = collect(1:10)
	
	myplot(x,y)
	myplot(x,y;color="blue")
	myplot(x,y;style="dashed",width=3)
end

# ╔═╡ ba247887-6163-4052-a075-f0ea93205f3e
note(md"The semicolon is necessary when defining the keyword arguments but they are not necessary when calling it.")

# ╔═╡ c5243da6-e25c-43ba-ae4a-be8ee3f3e67c
md"When having a chain of functions it can be convenient to bundle keyword arguments and pass them further. This can be done like this:"

# ╔═╡ c79bbdd6-20e2-4669-a3fd-ef6632125b7b
with_terminal() do
	function bar(x; myarg=10, kargs...)
		@info myarg, kargs
	end
	
	function foo(x; kargs...)
		bar(x; kargs...)
	end
	
	
	# Lets call it without keyword arguments
	foo(0)
	# Lets change myarg
	foo(0, myarg=5)
	# And now lets pass another keyword argument 
	foo(0, myarg=5, mysecondarg=13)
end

# ╔═╡ 7d0300b9-bd0d-40ec-ba73-f2c2150b129e
md"In practice this means that it is possible to tunnel arguments into low-level functions without the need to change the high-level interface. "

# ╔═╡ 7e0cc06a-e506-4679-a063-dd9396e5370e
md"(more information: [Julia Documentation]
(https://docs.julialang.org/en/v1/manual/functions/#Keyword-Arguments))"

# ╔═╡ f6fcf5b4-90ca-11eb-0650-87d8b0fef88b
md"### Evaluation Strategy

An interesting question when using functions is what evaluation strategy is used. Julia has call-by-value semantics, which means that a copy is being made when passing a value to a function. It also means that the arguments are first evaluated before they are passed to the function.
"

# ╔═╡ a5d02002-90cb-11eb-2e9b-ef925998dd8b
function evaluationStrategy(a)
  a = 7 
  return
end

# ╔═╡ dddb1c0e-90cb-11eb-1fdd-c97b40c87ef9
with_terminal() do
	y = 3
	evaluationStrategy(y)
	@info y
end

# ╔═╡ 240744fa-90cc-11eb-3193-55bd4f2dbdcc
md"Hence, one can see that a copy of `y` has been made when passing it to the function `evaluationStrategy`.

However, for many types in Julia (so-called mutable types) the variable actually stores only a reference to the data. This is similar like in C where a pointer is often used to store a reference to a complex data type. Lets see this in action using the array datatype.
"

# ╔═╡ cf2214aa-90cc-11eb-30b7-d350c3d6ba58
function evaluationStrategy2(a)
  a[1] = 7 
  return
end

# ╔═╡ e99bba02-90cc-11eb-1673-879c2f2fd4ef
with_terminal() do
	z = [3]
	evaluationStrategy2(z)
	@info z
end

# ╔═╡ f95dfd42-90cc-11eb-2fa8-07335bfa0588
md"You can see, what has changed is the data behind the reference `z`, its not `z` itself."

# ╔═╡ a0bbd8bf-d3c0-4509-8fae-77c7ad65d70e
md"### Methods

Functions in Julia map arguments to return values, whereas the *type* of the arguments matters. Thus we can have: 
"

# ╔═╡ ae3aa6a3-8c30-4f54-a84f-ba9edbc4c748
begin
	foobar(x::Int) = x
	foobar(x::Float64) = -x
end

# ╔═╡ 362ee388-0bf9-4100-9ca1-2b527df8a96c
md"Lets call them:"

# ╔═╡ 8e29da94-b262-4146-aa76-6d23723d6482
with_terminal() do
	@info foobar(1)
	@info foobar(1.0)
end

# ╔═╡ c31c19f8-d2a1-47de-9331-55600842fc99
md"So it seems that Julia allows the same function name for different argument types. Both functions are still related and we call `foobar` a *method*. You can also list all functions associated with it:"

# ╔═╡ b4bc673d-d8c8-4e09-afa6-a4931c9b41d4
with_terminal() do
	@info methods(foobar)
end

# ╔═╡ d8012453-0aae-40c2-a20d-d37b100795d2
md"We will not go deeper into methods at this point but will have a dedicated lecture on this important topic."

# ╔═╡ c2b497d0-913d-11eb-05d6-7955d6ec6c74
md"## Arithmetic Operators

Julia offers the well known arithmetic operators `+,-,*,/,^`. So you can do simple calculations:
"

# ╔═╡ 74d68aae-913e-11eb-232f-13ad436686ae
md"In Julia operators are nothing special but regular functions. One can either use the infix notation
```julia
3 + 4
```
or the prefix notation
```julia
+(3,4)
```
The later is important if you want to implement an operator for a new type. In other programming languages this is called `operator overloading`, in Julia the transformation from infix to postfix notation is just syntactic sugar and done during code lowering.

Here is an example of implementing the addition for a new type:
"

# ╔═╡ ea7bdc18-913f-11eb-0272-f1a5ad0e721c
begin
	import Base.+ # we need to import a method if we want to add a new function
	
	mutable struct MySuperInt # ignore this definition for the moment
		val::Int
	end
	
	function +(a::MySuperInt, b::MySuperInt)
	  return MySuperInt(a.val + b.val)
	end
	
	MySuperInt(3) + MySuperInt(4) 
end

# ╔═╡ 58cfa7d2-913e-11eb-05d6-ff28c8c8498d
ϕ = 3 * 4 + 17 + 2^4

# ╔═╡ bb557874-9147-11eb-1668-39a5cba21dc1
md"## Ranges and Vectors

We discuss array/vectors/ranges in more detail in the upcoming lectures. At this point we just want to briefly introduce some basics. 

### Vectors

Vectors can be created and handled like this:
"

# ╔═╡ d504529a-9147-11eb-38ad-2f40c72edff4
with_terminal() do
	a = [1,2,3,4] # this is a length-4 vector
	
	@show a
	@show a[3] # we access elements using square brackets
	a[3] = -10 # setting a value
	@show a
	nothing
end

# ╔═╡ 37092042-9148-11eb-338b-0d34614df7fb
md"At this point you should already have seen that Julia uses 1-based indexing like Matlab."

# ╔═╡ 5841b1a2-9148-11eb-08ab-4d3961a05288
md"### Ranges

Ranges are vector-like objects, where the elements are not stored in memory but are created on the fly. We use them extensively for indexing and iteration. Here are some examples ranges:
"

# ╔═╡ ca152ffc-9157-11eb-08f2-ad9a68f6b6d9
with_terminal() do
	a = 1:4
	
	@show a
	@show a[3] # we access elements using square brackets
	@show collect(a) # convert a range into a dense vector using collect 
	@show collect(1:2:9) # every second element
	nothing
end

# ╔═╡ 6d157b32-9159-11eb-0af7-d5065c5f93a6
md"In addition to the special range syntax, there is also a function syntax that is in many cases more flexible:"

# ╔═╡ 87b344da-9159-11eb-206b-afa666055114
with_terminal() do

	@show collect( range(1, 4, length=5) )
	@show collect( range(1, length=5, step=0.5) )
    nothing
end

# ╔═╡ c3e653c8-9147-11eb-0df1-39b2adca1a2f
md"## Control Flow

Next we discuss the control flow expressions that are available in Julia
"

# ╔═╡ a693f6d8-9148-11eb-1ad1-a151deaac3f3
md"### Conditional Evaluation

Quite often one wants to execute different code path depending on the *value* of an expression. This is done using the well-known `if` statement:
"

# ╔═╡ 02e65534-9178-11eb-191e-41a55663e13b
with_terminal() do
	
	students = ["Marie", "Peter", "Klaus", "Waltraut"]
	teachers = ["Hugo", "Jane"]
	
	s = "Peter"
	if s in students
		@show "$s is a student!"
	elseif s in teachers
	    @show "$s is a teacher!"
	else
		@show "$s is neither a teacher nor a student"
	end
	nothing
end

# ╔═╡ ad371550-9178-11eb-3163-278104dd8f22
md"Also the ternary operator is available:"

# ╔═╡ d15c5d0a-9178-11eb-0bdd-d7fac5214a12
with_terminal() do
	
	a = true
	
	b = a ? 14 : 7
	c = !a ? 14 : 7
	
	@show b,c
	nothing
end

# ╔═╡ 6c14d16a-917b-11eb-2a88-6bb1acc7b54d
md"One interesting question for `if` statements is whether they have their own scope. Lets consider the following C code:

```
if( ... ) {
  int i = 10;
} else {
  int i = 20
}

// do something with i
```

This will not work, since the variable `i` is only locally declared in the if statement. Instead one needs to first declare the variable: 


```
int i;
if( ... ) {
  i = 10;
} else {
  i = 20
}

// do something with i
```
"

# ╔═╡ 2f73f10a-917c-11eb-2ab7-f52a22898d60
md"In Julia `if` statements do not have their own scope and thus we can do:"

# ╔═╡ 4680264a-917c-11eb-38d9-69a3b5edbe0c
with_terminal() do
	a = true
	
	if a
		i = 10
	else
		i = 20
	end
	
	@show i
	nothing
end

# ╔═╡ 5b72463a-9177-11eb-002d-8fccb44d6cce
md"### Short-Circuit Evaluation

The boolean operations `&&` and `||` are usually used within `if` statements:

"

# ╔═╡ a2044a4e-9179-11eb-24c7-b18c52aa2201
with_terminal() do
	
	a = [] # I am an empty array
	
	if !isempty(a) && a[14] == 7
		println("do something with $a")
	else
	    println("don't something with $a")
	end
end

# ╔═╡ a2df0952-9179-11eb-33b0-470431612d5b
md"
You can see that the second part in the `if` clause is not evaluated since `a[14]` actually would throw an error. This is because of the following rules: 

* In the expression `a && b`, the subexpression `b` is only evaluated if `a` evaluates to true.
* In the expression `a || b` the subexpression `b` is only evaluated if `a` evaluates to false.

But actually this is not limited to `if` statements, one can actually also use this for regular control flow:
"

# ╔═╡ 965d94b0-917a-11eb-126c-e38b079e5225
with_terminal() do
	
	debug = true
	
	debug && println("This is only shown in debug mode!")
	
    error = false
	
	error || println("No error happened, we can proceed!")
end

# ╔═╡ 0877c6ba-917b-11eb-15da-2f65cd32c8b7
note(md"In many cases the code remains actually more readable when sticking to simple `if` statements.")

# ╔═╡ b233611a-9148-11eb-3e0f-29871afbf62e
md"### Repeated Evaluation
Another very important control flow mechanism is repeated evaluation or loops. They allow to repeat code fragments for a certain number of times. Julia supports two types, `for` loops and `while` loops. Lets see both in action:
"

# ╔═╡ bbb5798a-9148-11eb-0f24-95a8f57181b0
with_terminal() do
  for i=1:4
	print("$i ")
  end
	
	println("")

  l = 4
  while l< 10
	print("$l ")
	l += 2
  end
	
end

# ╔═╡ 3fa7226e-299c-4b60-b78e-4f22839a3418
md"Both should be pretty clear. `for` loops are very powerful in Julia and can be used to loop over containers quite easily."

# ╔═╡ 34ae7ce1-495f-4c89-b5f6-5e5ee9a5b8a2
with_terminal() do
  global names = ["Hans", "Judith", "Peter", "Angelina"]
	
  for name in names
    @info name		
  end
end

# ╔═╡ 67808046-cfd8-4c0e-8412-50a8f869dd6c
md"Now what is if you need both the element and the index in the `for` loop?"

# ╔═╡ 47b6cf42-28ae-457d-af21-0bfc6fe58504
with_terminal() do
  for (i,name) in enumerate(names)
    @info name, i		
  end
end

# ╔═╡ 17cc99a1-812f-40de-a057-8d913662d160
md"Thats cool, isn't it?"

# ╔═╡ 722695de-6735-11eb-1fa2-f3acb00898dc
md"
## Custom and Composite Types

In many situations one wants to create new types that group several values together. In Julia this is done by `structs`. For instance we can create a struct `Student` that has two members `name` and `age`:
"

# ╔═╡ dfb9cb84-6735-11eb-28e7-6112c7c78cdf
struct Student
	name::String
	age::Int
end

# ╔═╡ 0030fa2c-6736-11eb-354b-05a6d2a58fd5
md"Lets create a student"

# ╔═╡ 10a19182-6736-11eb-0034-71e41dc49f5f
s = Student("Tobi", 21)

# ╔═╡ 208dcb92-6736-11eb-1075-ef4230aafe3e
md"We can access the members using the dot syntax"

# ╔═╡ 370b5416-6736-11eb-2b6e-c3a96ab2979f
s.name

# ╔═╡ 43909a02-6736-11eb-09c0-dd1b8d80d0a4
s.age

# ╔═╡ 603d715e-6736-11eb-324e-8d5698fb7200
md"
#### Immutability

The struct that we discussed so far is a so-called *immutable* struct. Immutable means that we cannot change it
"

# ╔═╡ 8f4b9abe-6736-11eb-21cf-1564d4ab6047
s.name = "Franz"

# ╔═╡ 9b0585b8-6736-11eb-386e-83fae463664b
md"
It might not be clear at this point what the advantage of immutability is but you can basically think of this as a hint for the compiler that allows for several optimizations that improve the execution speed. 
"

# ╔═╡ e7c7ef6c-6736-11eb-02cd-0d990b0aea52
md"
If you need mutability just use a mutable struct
"

# ╔═╡ 35857026-6737-11eb-0d84-35926fcba6e6
mutable struct MutableStudent
	name::String
	age::Int
end

# ╔═╡ 56934680-6737-11eb-38df-09c036804ad7
c = MutableStudent("Karl", 40)

# ╔═╡ 6d489466-6737-11eb-281d-7dd70b29b50e
c.age = 10

# ╔═╡ 7f33c86c-6737-11eb-0552-19ff450139b3
md"What is important for `structs` and `mutable structs` is that you specify the types. If you do not do this, it is possible that members can change their type during lifetime, which will prevent emitting efficient code."

# ╔═╡ 423711c0-6f06-11eb-3440-b5eb20e3305c
with_terminal() do
	@info isimmutable(s)
	@info isimmutable(c)
end

# ╔═╡ 2a20cf98-9160-11eb-2e36-2963f159811a
md"## Packages and Modules

When developing a large codebase it is important to structure the code in a way that common functionality is grouped together. In Julia this is done using `modules`. At this point in the lecture we will not go into details of modules.

In Julia a package is a more general concept. A package can consist of one or several modules that is grouped together such that other people can use it. Packages in Julia usually correspond to a git repository (hosted e.g. at GitHub). Here is an example package *NFFT.jl* that allows for carrying a generalization of the Fast Fourier Transform (FFT):
"

# ╔═╡ f5daef7a-9164-11eb-1dba-01580661c18a
PlutoUI.LocalResource("img/NFFT.png", :width => 900)

# ╔═╡ 3521ac82-9165-11eb-29b7-bdd187f0264a
md"One can see that the files and folders are structured in a certain way. Furthermore one can see some tooling that is offered in addition plain git:

* Continuous Integration (CI)
* Automatic generation of documentation
* Code coverage"

# ╔═╡ 4d8c448e-9166-11eb-1741-3d968ac8f36d
md"### Package Ecosystem

Julia has a very large package ecosystem and thus many things to not need to be implemented by yourself. For instance:

* You need to do Spline interpolation on arbitrary dimensioned arrays? Just use [Interpolations.jl](https://github.com/JuliaMath/Interpolations.jl)
* Need to solve differential equations? Just use [DifferentialEquations.jl](https://github.com/SciML/DifferentialEquations.jl)
* You need to develop a neural network? Just use [Flux.jl](https://github.com/FluxML/Flux.jl)

It should be noted that the entire Julia ecosystem is, similar to wikipedia, a collection of open source projects. Thus, the quality of packages varies greatly. But there is a certain fraction of high-quality packages out there that are already more sophisticated than similar packages in Python or Matlab. If you want to explore the package ecosystem you can visit [JuliaHub](https://juliahub.com).

"

# ╔═╡ ad734b64-9165-11eb-28b7-b5f91838c755
md"### Using Packages

At this point, again, we do not detail, what a package is exactly. Instead we just want to make you aware that they exist and how they can be used.

To install a package you go into package mode by entering `]` and then enter (if you want to install *Interpolations.jl*):

```julia
pkg> add Interpolations
```

That's it. After you have installed it you need to load the package. This is done in Julia by:

```julia
using Interpolations
```
Then all functions exported by *Interpolations* are brought into the current namespace and can be used.
"

# ╔═╡ 2c8700c0-9135-11eb-11b7-3163d4665332
md"## Visualization

There are various packages in Julia to perform visualization, i.e. plotting of functions or displaying images. Two popular packages you should be aware of are *Plots.jl* and *PyPlot.jl*. We will mostly use *Plots.jl*.
"

# ╔═╡ d7730092-9167-11eb-2091-8d6f0f3c8e22
begin
	t = range(0, 2π, length=100)
	u = sin.(t)
	p = plot(t, u, label="A nice sine function", lw=3)
	xlabel!(p, "t / s")
	ylabel!(p, "u / V")
end

# ╔═╡ 00000000-0000-0000-0000-000000000001
PLUTO_PROJECT_TOML_CONTENTS = """
[deps]
LinearAlgebra = "37e2e46d-f89d-539d-b4ee-838fcccc9c8e"
Pkg = "44cfe95a-1eb2-52ea-b672-e2afdf69b78f"
Plots = "91a5bcdd-55d7-5caf-9e0b-520d859cae80"
PlutoUI = "7f904dfe-b85e-4ff6-b463-dae2292396a8"
TestImages = "5e47fb64-e119-507b-a336-dd2b206d9990"

[compat]
Plots = "~1.35.5"
PlutoUI = "~0.7.48"
TestImages = "~1.7.1"
"""

# ╔═╡ 00000000-0000-0000-0000-000000000002
PLUTO_MANIFEST_TOML_CONTENTS = """
# This file is machine-generated - editing it directly is not advised

julia_version = "1.8.0"
manifest_format = "2.0"
project_hash = "04f9493428fc3e8961872de579e16202f697e245"

[[deps.AbstractFFTs]]
deps = ["ChainRulesCore", "LinearAlgebra"]
git-tree-sha1 = "69f7020bd72f069c219b5e8c236c1fa90d2cb409"
uuid = "621f4979-c628-5d54-868e-fcf4e3e8185c"
version = "1.2.1"

[[deps.AbstractPlutoDingetjes]]
deps = ["Pkg"]
git-tree-sha1 = "8eaf9f1b4921132a4cff3f36a1d9ba923b14a481"
uuid = "6e696c72-6542-2067-7265-42206c756150"
version = "1.1.4"

[[deps.Adapt]]
deps = ["LinearAlgebra"]
git-tree-sha1 = "195c5505521008abea5aee4f96930717958eac6f"
uuid = "79e6a3ab-5dfb-504d-930d-738a2a938a0e"
version = "3.4.0"

[[deps.ArgTools]]
uuid = "0dad84c5-d112-42e6-8d28-ef12dabb789f"
version = "1.1.1"

[[deps.Artifacts]]
uuid = "56f22d72-fd6d-98f1-02f0-08ddc0907c33"

[[deps.AxisArrays]]
deps = ["Dates", "IntervalSets", "IterTools", "RangeArrays"]
git-tree-sha1 = "1dd4d9f5beebac0c03446918741b1a03dc5e5788"
uuid = "39de3d68-74b9-583c-8d2d-e117c070f3a9"
version = "0.4.6"

[[deps.Base64]]
uuid = "2a0f44e3-6c83-55bd-87e4-b1978d98bd5f"

[[deps.BitFlags]]
git-tree-sha1 = "84259bb6172806304b9101094a7cc4bc6f56dbc6"
uuid = "d1d4a3ce-64b1-5f1a-9ba4-7e7e69966f35"
version = "0.1.5"

[[deps.Bzip2_jll]]
deps = ["Artifacts", "JLLWrappers", "Libdl", "Pkg"]
git-tree-sha1 = "19a35467a82e236ff51bc17a3a44b69ef35185a2"
uuid = "6e34b625-4abd-537c-b88f-471c36dfa7a0"
version = "1.0.8+0"

[[deps.CEnum]]
git-tree-sha1 = "eb4cb44a499229b3b8426dcfb5dd85333951ff90"
uuid = "fa961155-64e5-5f13-b03f-caf6b980ea82"
version = "0.4.2"

[[deps.Cairo_jll]]
deps = ["Artifacts", "Bzip2_jll", "Fontconfig_jll", "FreeType2_jll", "Glib_jll", "JLLWrappers", "LZO_jll", "Libdl", "Pixman_jll", "Pkg", "Xorg_libXext_jll", "Xorg_libXrender_jll", "Zlib_jll", "libpng_jll"]
git-tree-sha1 = "4b859a208b2397a7a623a03449e4636bdb17bcf2"
uuid = "83423d85-b0ee-5818-9007-b63ccbeb887a"
version = "1.16.1+1"

[[deps.ChainRulesCore]]
deps = ["Compat", "LinearAlgebra", "SparseArrays"]
git-tree-sha1 = "e7ff6cadf743c098e08fca25c91103ee4303c9bb"
uuid = "d360d2e6-b24c-11e9-a2a3-2a2ae2dbcce4"
version = "1.15.6"

[[deps.ChangesOfVariables]]
deps = ["ChainRulesCore", "LinearAlgebra", "Test"]
git-tree-sha1 = "38f7a08f19d8810338d4f5085211c7dfa5d5bdd8"
uuid = "9e997f8a-9a97-42d5-a9f1-ce6bfc15e2c0"
version = "0.1.4"

[[deps.CodecZlib]]
deps = ["TranscodingStreams", "Zlib_jll"]
git-tree-sha1 = "ded953804d019afa9a3f98981d99b33e3db7b6da"
uuid = "944b1d66-785c-5afd-91f1-9de20f533193"
version = "0.7.0"

[[deps.ColorSchemes]]
deps = ["ColorTypes", "ColorVectorSpace", "Colors", "FixedPointNumbers", "Random"]
git-tree-sha1 = "1fd869cc3875b57347f7027521f561cf46d1fcd8"
uuid = "35d6a980-a343-548e-a6ea-1d62b119f2f4"
version = "3.19.0"

[[deps.ColorTypes]]
deps = ["FixedPointNumbers", "Random"]
git-tree-sha1 = "eb7f0f8307f71fac7c606984ea5fb2817275d6e4"
uuid = "3da002f7-5984-5a60-b8a6-cbb66c0b333f"
version = "0.11.4"

[[deps.ColorVectorSpace]]
deps = ["ColorTypes", "FixedPointNumbers", "LinearAlgebra", "SpecialFunctions", "Statistics", "TensorCore"]
git-tree-sha1 = "d08c20eef1f2cbc6e60fd3612ac4340b89fea322"
uuid = "c3611d14-8923-5661-9e6a-0046d554d3a4"
version = "0.9.9"

[[deps.Colors]]
deps = ["ColorTypes", "FixedPointNumbers", "Reexport"]
git-tree-sha1 = "417b0ed7b8b838aa6ca0a87aadf1bb9eb111ce40"
uuid = "5ae59095-9a9b-59fe-a467-6f913c188581"
version = "0.12.8"

[[deps.Compat]]
deps = ["Dates", "LinearAlgebra", "UUIDs"]
git-tree-sha1 = "3ca828fe1b75fa84b021a7860bd039eaea84d2f2"
uuid = "34da2185-b29b-5c13-b0c7-acf172513d20"
version = "4.3.0"

[[deps.CompilerSupportLibraries_jll]]
deps = ["Artifacts", "Libdl"]
uuid = "e66e0078-7015-5450-92f7-15fbd957f2ae"
version = "0.5.2+0"

[[deps.Contour]]
git-tree-sha1 = "d05d9e7b7aedff4e5b51a029dced05cfb6125781"
uuid = "d38c429a-6771-53c6-b99e-75d170b6e991"
version = "0.6.2"

[[deps.DataAPI]]
git-tree-sha1 = "46d2680e618f8abd007bce0c3026cb0c4a8f2032"
uuid = "9a962f9c-6df0-11e9-0e5d-c546b8b5ee8a"
version = "1.12.0"

[[deps.DataStructures]]
deps = ["Compat", "InteractiveUtils", "OrderedCollections"]
git-tree-sha1 = "d1fff3a548102f48987a52a2e0d114fa97d730f0"
uuid = "864edb3b-99cc-5e75-8d2d-829cb0a9cfe8"
version = "0.18.13"

[[deps.Dates]]
deps = ["Printf"]
uuid = "ade2ca70-3891-5945-98fb-dc099432e06a"

[[deps.DelimitedFiles]]
deps = ["Mmap"]
uuid = "8bb1440f-4735-579b-a4ab-409b98df4dab"

[[deps.Distances]]
deps = ["LinearAlgebra", "SparseArrays", "Statistics", "StatsAPI"]
git-tree-sha1 = "3258d0659f812acde79e8a74b11f17ac06d0ca04"
uuid = "b4f34e82-e78d-54a5-968a-f98e89d6e8f7"
version = "0.10.7"

[[deps.Distributed]]
deps = ["Random", "Serialization", "Sockets"]
uuid = "8ba89e20-285c-5b6f-9357-94700520ee1b"

[[deps.DocStringExtensions]]
deps = ["LibGit2"]
git-tree-sha1 = "c36550cb29cbe373e95b3f40486b9a4148f89ffd"
uuid = "ffbed154-4ef7-542d-bbb7-c09d3a79fcae"
version = "0.9.2"

[[deps.Downloads]]
deps = ["ArgTools", "FileWatching", "LibCURL", "NetworkOptions"]
uuid = "f43a241f-c20a-4ad4-852c-f6b1247861c6"
version = "1.6.0"

[[deps.Expat_jll]]
deps = ["Artifacts", "JLLWrappers", "Libdl", "Pkg"]
git-tree-sha1 = "bad72f730e9e91c08d9427d5e8db95478a3c323d"
uuid = "2e619515-83b5-522b-bb60-26c02a35a201"
version = "2.4.8+0"

[[deps.FFMPEG]]
deps = ["FFMPEG_jll"]
git-tree-sha1 = "b57e3acbe22f8484b4b5ff66a7499717fe1a9cc8"
uuid = "c87230d0-a227-11e9-1b43-d7ebe4e7570a"
version = "0.4.1"

[[deps.FFMPEG_jll]]
deps = ["Artifacts", "Bzip2_jll", "FreeType2_jll", "FriBidi_jll", "JLLWrappers", "LAME_jll", "Libdl", "Ogg_jll", "OpenSSL_jll", "Opus_jll", "PCRE2_jll", "Pkg", "Zlib_jll", "libaom_jll", "libass_jll", "libfdk_aac_jll", "libvorbis_jll", "x264_jll", "x265_jll"]
git-tree-sha1 = "74faea50c1d007c85837327f6775bea60b5492dd"
uuid = "b22a6f82-2f65-5046-a5b2-351ab43fb4e5"
version = "4.4.2+2"

[[deps.FileIO]]
deps = ["Pkg", "Requires", "UUIDs"]
git-tree-sha1 = "7be5f99f7d15578798f338f5433b6c432ea8037b"
uuid = "5789e2e9-d7fb-5bc7-8068-2c6fae9b9549"
version = "1.16.0"

[[deps.FileWatching]]
uuid = "7b1f6079-737a-58dc-b8bc-7a2ca5c1b5ee"

[[deps.FixedPointNumbers]]
deps = ["Statistics"]
git-tree-sha1 = "335bfdceacc84c5cdf16aadc768aa5ddfc5383cc"
uuid = "53c48c17-4a7d-5ca2-90c5-79b7896eea93"
version = "0.8.4"

[[deps.Fontconfig_jll]]
deps = ["Artifacts", "Bzip2_jll", "Expat_jll", "FreeType2_jll", "JLLWrappers", "Libdl", "Libuuid_jll", "Pkg", "Zlib_jll"]
git-tree-sha1 = "21efd19106a55620a188615da6d3d06cd7f6ee03"
uuid = "a3f928ae-7b40-5064-980b-68af3947d34b"
version = "2.13.93+0"

[[deps.Formatting]]
deps = ["Printf"]
git-tree-sha1 = "8339d61043228fdd3eb658d86c926cb282ae72a8"
uuid = "59287772-0a20-5a39-b81b-1366585eb4c0"
version = "0.4.2"

[[deps.FreeType2_jll]]
deps = ["Artifacts", "Bzip2_jll", "JLLWrappers", "Libdl", "Pkg", "Zlib_jll"]
git-tree-sha1 = "87eb71354d8ec1a96d4a7636bd57a7347dde3ef9"
uuid = "d7e528f0-a631-5988-bf34-fe36492bcfd7"
version = "2.10.4+0"

[[deps.FriBidi_jll]]
deps = ["Artifacts", "JLLWrappers", "Libdl", "Pkg"]
git-tree-sha1 = "aa31987c2ba8704e23c6c8ba8a4f769d5d7e4f91"
uuid = "559328eb-81f9-559d-9380-de523a88c83c"
version = "1.0.10+0"

[[deps.GLFW_jll]]
deps = ["Artifacts", "JLLWrappers", "Libdl", "Libglvnd_jll", "Pkg", "Xorg_libXcursor_jll", "Xorg_libXi_jll", "Xorg_libXinerama_jll", "Xorg_libXrandr_jll"]
git-tree-sha1 = "d972031d28c8c8d9d7b41a536ad7bb0c2579caca"
uuid = "0656b61e-2033-5cc2-a64a-77c0f6c09b89"
version = "3.3.8+0"

[[deps.GR]]
deps = ["Base64", "DelimitedFiles", "GR_jll", "HTTP", "JSON", "Libdl", "LinearAlgebra", "Pkg", "Preferences", "Printf", "Random", "Serialization", "Sockets", "Test", "UUIDs"]
git-tree-sha1 = "00a9d4abadc05b9476e937a5557fcce476b9e547"
uuid = "28b8d3ca-fb5f-59d9-8090-bfdbd6d07a71"
version = "0.69.5"

[[deps.GR_jll]]
deps = ["Artifacts", "Bzip2_jll", "Cairo_jll", "FFMPEG_jll", "Fontconfig_jll", "GLFW_jll", "JLLWrappers", "JpegTurbo_jll", "Libdl", "Libtiff_jll", "Pixman_jll", "Pkg", "Qt5Base_jll", "Zlib_jll", "libpng_jll"]
git-tree-sha1 = "bc9f7725571ddb4ab2c4bc74fa397c1c5ad08943"
uuid = "d2c73de3-f751-5644-a686-071e5b155ba9"
version = "0.69.1+0"

[[deps.Gettext_jll]]
deps = ["Artifacts", "CompilerSupportLibraries_jll", "JLLWrappers", "Libdl", "Libiconv_jll", "Pkg", "XML2_jll"]
git-tree-sha1 = "9b02998aba7bf074d14de89f9d37ca24a1a0b046"
uuid = "78b55507-aeef-58d4-861c-77aaff3498b1"
version = "0.21.0+0"

[[deps.Ghostscript_jll]]
deps = ["Artifacts", "JLLWrappers", "Libdl", "Pkg"]
git-tree-sha1 = "78e2c69783c9753a91cdae88a8d432be85a2ab5e"
uuid = "61579ee1-b43e-5ca0-a5da-69d92c66a64b"
version = "9.55.0+0"

[[deps.Glib_jll]]
deps = ["Artifacts", "Gettext_jll", "JLLWrappers", "Libdl", "Libffi_jll", "Libiconv_jll", "Libmount_jll", "PCRE2_jll", "Pkg", "Zlib_jll"]
git-tree-sha1 = "fb83fbe02fe57f2c068013aa94bcdf6760d3a7a7"
uuid = "7746bdde-850d-59dc-9ae8-88ece973131d"
version = "2.74.0+1"

[[deps.Graphics]]
deps = ["Colors", "LinearAlgebra", "NaNMath"]
git-tree-sha1 = "d61890399bc535850c4bf08e4e0d3a7ad0f21cbd"
uuid = "a2bd30eb-e257-5431-a919-1863eab51364"
version = "1.1.2"

[[deps.Graphite2_jll]]
deps = ["Artifacts", "JLLWrappers", "Libdl", "Pkg"]
git-tree-sha1 = "344bf40dcab1073aca04aa0df4fb092f920e4011"
uuid = "3b182d85-2403-5c21-9c21-1e1f0cc25472"
version = "1.3.14+0"

[[deps.Grisu]]
git-tree-sha1 = "53bb909d1151e57e2484c3d1b53e19552b887fb2"
uuid = "42e2da0e-8278-4e71-bc24-59509adca0fe"
version = "1.0.2"

[[deps.HTTP]]
deps = ["Base64", "CodecZlib", "Dates", "IniFile", "Logging", "LoggingExtras", "MbedTLS", "NetworkOptions", "OpenSSL", "Random", "SimpleBufferStream", "Sockets", "URIs", "UUIDs"]
git-tree-sha1 = "a97d47758e933cd5fe5ea181d178936a9fc60427"
uuid = "cd3eb016-35fb-5094-929b-558a96fad6f3"
version = "1.5.1"

[[deps.HarfBuzz_jll]]
deps = ["Artifacts", "Cairo_jll", "Fontconfig_jll", "FreeType2_jll", "Glib_jll", "Graphite2_jll", "JLLWrappers", "Libdl", "Libffi_jll", "Pkg"]
git-tree-sha1 = "129acf094d168394e80ee1dc4bc06ec835e510a3"
uuid = "2e76f6c2-a576-52d4-95c1-20adfe4de566"
version = "2.8.1+1"

[[deps.Hyperscript]]
deps = ["Test"]
git-tree-sha1 = "8d511d5b81240fc8e6802386302675bdf47737b9"
uuid = "47d2ed2b-36de-50cf-bf87-49c2cf4b8b91"
version = "0.0.4"

[[deps.HypertextLiteral]]
deps = ["Tricks"]
git-tree-sha1 = "c47c5fa4c5308f27ccaac35504858d8914e102f9"
uuid = "ac1192a8-f4b3-4bfe-ba22-af5b92cd3ab2"
version = "0.9.4"

[[deps.IOCapture]]
deps = ["Logging", "Random"]
git-tree-sha1 = "f7be53659ab06ddc986428d3a9dcc95f6fa6705a"
uuid = "b5f81e59-6552-4d32-b1f0-c071b021bf89"
version = "0.2.2"

[[deps.ImageCore]]
deps = ["AbstractFFTs", "ColorVectorSpace", "Colors", "FixedPointNumbers", "Graphics", "MappedArrays", "MosaicViews", "OffsetArrays", "PaddedViews", "Reexport"]
git-tree-sha1 = "acf614720ef026d38400b3817614c45882d75500"
uuid = "a09fc81d-aa75-5fe9-8630-4744c3626534"
version = "0.9.4"

[[deps.ImageIO]]
deps = ["FileIO", "IndirectArrays", "JpegTurbo", "LazyModules", "Netpbm", "OpenEXR", "PNGFiles", "QOI", "Sixel", "TiffImages", "UUIDs"]
git-tree-sha1 = "342f789fd041a55166764c351da1710db97ce0e0"
uuid = "82e4d734-157c-48bb-816b-45c225c6df19"
version = "0.6.6"

[[deps.ImageMagick]]
deps = ["FileIO", "ImageCore", "ImageMagick_jll", "InteractiveUtils", "Libdl", "Pkg", "Random"]
git-tree-sha1 = "5bc1cb62e0c5f1005868358db0692c994c3a13c6"
uuid = "6218d12a-5da1-5696-b52f-db25d2ecc6d1"
version = "1.2.1"

[[deps.ImageMagick_jll]]
deps = ["Artifacts", "Ghostscript_jll", "JLLWrappers", "JpegTurbo_jll", "Libdl", "Libtiff_jll", "Pkg", "Zlib_jll", "libpng_jll"]
git-tree-sha1 = "124626988534986113cfd876e3093e4a03890f58"
uuid = "c73af94c-d91f-53ed-93a7-00f77d67a9d7"
version = "6.9.12+3"

[[deps.Imath_jll]]
deps = ["Artifacts", "JLLWrappers", "Libdl", "Pkg"]
git-tree-sha1 = "87f7662e03a649cffa2e05bf19c303e168732d3e"
uuid = "905a6f67-0a94-5f89-b386-d35d92009cd1"
version = "3.1.2+0"

[[deps.IndirectArrays]]
git-tree-sha1 = "012e604e1c7458645cb8b436f8fba789a51b257f"
uuid = "9b13fd28-a010-5f03-acff-a1bbcff69959"
version = "1.0.0"

[[deps.Inflate]]
git-tree-sha1 = "5cd07aab533df5170988219191dfad0519391428"
uuid = "d25df0c9-e2be-5dd7-82c8-3ad0b3e990b9"
version = "0.1.3"

[[deps.IniFile]]
git-tree-sha1 = "f550e6e32074c939295eb5ea6de31849ac2c9625"
uuid = "83e8ac13-25f8-5344-8a64-a9f2b223428f"
version = "0.5.1"

[[deps.InteractiveUtils]]
deps = ["Markdown"]
uuid = "b77e0a4c-d291-57a0-90e8-8db25a27a240"

[[deps.IntervalSets]]
deps = ["Dates", "Random", "Statistics"]
git-tree-sha1 = "3f91cd3f56ea48d4d2a75c2a65455c5fc74fa347"
uuid = "8197267c-284f-5f27-9208-e0e47529a953"
version = "0.7.3"

[[deps.InverseFunctions]]
deps = ["Test"]
git-tree-sha1 = "49510dfcb407e572524ba94aeae2fced1f3feb0f"
uuid = "3587e190-3f89-42d0-90ee-14403ec27112"
version = "0.1.8"

[[deps.IrrationalConstants]]
git-tree-sha1 = "7fd44fd4ff43fc60815f8e764c0f352b83c49151"
uuid = "92d709cd-6900-40b7-9082-c6be49f344b6"
version = "0.1.1"

[[deps.IterTools]]
git-tree-sha1 = "fa6287a4469f5e048d763df38279ee729fbd44e5"
uuid = "c8e1da08-722c-5040-9ed9-7db0dc04731e"
version = "1.4.0"

[[deps.JLFzf]]
deps = ["Pipe", "REPL", "Random", "fzf_jll"]
git-tree-sha1 = "f377670cda23b6b7c1c0b3893e37451c5c1a2185"
uuid = "1019f520-868f-41f5-a6de-eb00f4b6a39c"
version = "0.1.5"

[[deps.JLLWrappers]]
deps = ["Preferences"]
git-tree-sha1 = "abc9885a7ca2052a736a600f7fa66209f96506e1"
uuid = "692b3bcd-3c85-4b1f-b108-f13ce0eb3210"
version = "1.4.1"

[[deps.JSON]]
deps = ["Dates", "Mmap", "Parsers", "Unicode"]
git-tree-sha1 = "3c837543ddb02250ef42f4738347454f95079d4e"
uuid = "682c06a0-de6a-54ab-a142-c8b1cf79cde6"
version = "0.21.3"

[[deps.JpegTurbo]]
deps = ["CEnum", "FileIO", "ImageCore", "JpegTurbo_jll", "TOML"]
git-tree-sha1 = "a77b273f1ddec645d1b7c4fd5fb98c8f90ad10a5"
uuid = "b835a17e-a41a-41e7-81f0-2f016b05efe0"
version = "0.1.1"

[[deps.JpegTurbo_jll]]
deps = ["Artifacts", "JLLWrappers", "Libdl", "Pkg"]
git-tree-sha1 = "b53380851c6e6664204efb2e62cd24fa5c47e4ba"
uuid = "aacddb02-875f-59d6-b918-886e6ef4fbf8"
version = "2.1.2+0"

[[deps.LAME_jll]]
deps = ["Artifacts", "JLLWrappers", "Libdl", "Pkg"]
git-tree-sha1 = "f6250b16881adf048549549fba48b1161acdac8c"
uuid = "c1c5ebd0-6772-5130-a774-d5fcae4a789d"
version = "3.100.1+0"

[[deps.LERC_jll]]
deps = ["Artifacts", "JLLWrappers", "Libdl", "Pkg"]
git-tree-sha1 = "bf36f528eec6634efc60d7ec062008f171071434"
uuid = "88015f11-f218-50d7-93a8-a6af411a945d"
version = "3.0.0+1"

[[deps.LZO_jll]]
deps = ["Artifacts", "JLLWrappers", "Libdl", "Pkg"]
git-tree-sha1 = "e5b909bcf985c5e2605737d2ce278ed791b89be6"
uuid = "dd4b983a-f0e5-5f8d-a1b7-129d4a5fb1ac"
version = "2.10.1+0"

[[deps.LaTeXStrings]]
git-tree-sha1 = "f2355693d6778a178ade15952b7ac47a4ff97996"
uuid = "b964fa9f-0449-5b57-a5c2-d3ea65f4040f"
version = "1.3.0"

[[deps.Latexify]]
deps = ["Formatting", "InteractiveUtils", "LaTeXStrings", "MacroTools", "Markdown", "OrderedCollections", "Printf", "Requires"]
git-tree-sha1 = "ab9aa169d2160129beb241cb2750ca499b4e90e9"
uuid = "23fbe1c1-3f47-55db-b15f-69d7ec21a316"
version = "0.15.17"

[[deps.LazyModules]]
git-tree-sha1 = "a560dd966b386ac9ae60bdd3a3d3a326062d3c3e"
uuid = "8cdb02fc-e678-4876-92c5-9defec4f444e"
version = "0.3.1"

[[deps.LibCURL]]
deps = ["LibCURL_jll", "MozillaCACerts_jll"]
uuid = "b27032c2-a3e7-50c8-80cd-2d36dbcbfd21"
version = "0.6.3"

[[deps.LibCURL_jll]]
deps = ["Artifacts", "LibSSH2_jll", "Libdl", "MbedTLS_jll", "Zlib_jll", "nghttp2_jll"]
uuid = "deac9b47-8bc7-5906-a0fe-35ac56dc84c0"
version = "7.84.0+0"

[[deps.LibGit2]]
deps = ["Base64", "NetworkOptions", "Printf", "SHA"]
uuid = "76f85450-5226-5b5a-8eaa-529ad045b433"

[[deps.LibSSH2_jll]]
deps = ["Artifacts", "Libdl", "MbedTLS_jll"]
uuid = "29816b5a-b9ab-546f-933c-edad1886dfa8"
version = "1.10.2+0"

[[deps.Libdl]]
uuid = "8f399da3-3557-5675-b5ff-fb832c97cbdb"

[[deps.Libffi_jll]]
deps = ["Artifacts", "JLLWrappers", "Libdl", "Pkg"]
git-tree-sha1 = "0b4a5d71f3e5200a7dff793393e09dfc2d874290"
uuid = "e9f186c6-92d2-5b65-8a66-fee21dc1b490"
version = "3.2.2+1"

[[deps.Libgcrypt_jll]]
deps = ["Artifacts", "JLLWrappers", "Libdl", "Libgpg_error_jll", "Pkg"]
git-tree-sha1 = "64613c82a59c120435c067c2b809fc61cf5166ae"
uuid = "d4300ac3-e22c-5743-9152-c294e39db1e4"
version = "1.8.7+0"

[[deps.Libglvnd_jll]]
deps = ["Artifacts", "JLLWrappers", "Libdl", "Pkg", "Xorg_libX11_jll", "Xorg_libXext_jll"]
git-tree-sha1 = "7739f837d6447403596a75d19ed01fd08d6f56bf"
uuid = "7e76a0d4-f3c7-5321-8279-8d96eeed0f29"
version = "1.3.0+3"

[[deps.Libgpg_error_jll]]
deps = ["Artifacts", "JLLWrappers", "Libdl", "Pkg"]
git-tree-sha1 = "c333716e46366857753e273ce6a69ee0945a6db9"
uuid = "7add5ba3-2f88-524e-9cd5-f83b8a55f7b8"
version = "1.42.0+0"

[[deps.Libiconv_jll]]
deps = ["Artifacts", "JLLWrappers", "Libdl", "Pkg"]
git-tree-sha1 = "42b62845d70a619f063a7da093d995ec8e15e778"
uuid = "94ce4f54-9a6c-5748-9c1c-f9c7231a4531"
version = "1.16.1+1"

[[deps.Libmount_jll]]
deps = ["Artifacts", "JLLWrappers", "Libdl", "Pkg"]
git-tree-sha1 = "9c30530bf0effd46e15e0fdcf2b8636e78cbbd73"
uuid = "4b2f31a3-9ecc-558c-b454-b3730dcb73e9"
version = "2.35.0+0"

[[deps.Libtiff_jll]]
deps = ["Artifacts", "JLLWrappers", "JpegTurbo_jll", "LERC_jll", "Libdl", "Pkg", "Zlib_jll", "Zstd_jll"]
git-tree-sha1 = "3eb79b0ca5764d4799c06699573fd8f533259713"
uuid = "89763e89-9b03-5906-acba-b20f662cd828"
version = "4.4.0+0"

[[deps.Libuuid_jll]]
deps = ["Artifacts", "JLLWrappers", "Libdl", "Pkg"]
git-tree-sha1 = "7f3efec06033682db852f8b3bc3c1d2b0a0ab066"
uuid = "38a345b3-de98-5d2b-a5d3-14cd9215e700"
version = "2.36.0+0"

[[deps.LinearAlgebra]]
deps = ["Libdl", "libblastrampoline_jll"]
uuid = "37e2e46d-f89d-539d-b4ee-838fcccc9c8e"

[[deps.LogExpFunctions]]
deps = ["ChainRulesCore", "ChangesOfVariables", "DocStringExtensions", "InverseFunctions", "IrrationalConstants", "LinearAlgebra"]
git-tree-sha1 = "94d9c52ca447e23eac0c0f074effbcd38830deb5"
uuid = "2ab3a3ac-af41-5b50-aa03-7779005ae688"
version = "0.3.18"

[[deps.Logging]]
uuid = "56ddb016-857b-54e1-b83d-db4d58db5568"

[[deps.LoggingExtras]]
deps = ["Dates", "Logging"]
git-tree-sha1 = "5d4d2d9904227b8bd66386c1138cf4d5ffa826bf"
uuid = "e6f89c97-d47a-5376-807f-9c37f3926c36"
version = "0.4.9"

[[deps.MIMEs]]
git-tree-sha1 = "65f28ad4b594aebe22157d6fac869786a255b7eb"
uuid = "6c6e2e6c-3030-632d-7369-2d6c69616d65"
version = "0.1.4"

[[deps.MacroTools]]
deps = ["Markdown", "Random"]
git-tree-sha1 = "42324d08725e200c23d4dfb549e0d5d89dede2d2"
uuid = "1914dd2f-81c6-5fcd-8719-6d5c9610ff09"
version = "0.5.10"

[[deps.MappedArrays]]
git-tree-sha1 = "e8b359ef06ec72e8c030463fe02efe5527ee5142"
uuid = "dbb5928d-eab1-5f90-85c2-b9b0edb7c900"
version = "0.4.1"

[[deps.Markdown]]
deps = ["Base64"]
uuid = "d6f4376e-aef5-505a-96c1-9c027394607a"

[[deps.MbedTLS]]
deps = ["Dates", "MbedTLS_jll", "MozillaCACerts_jll", "Random", "Sockets"]
git-tree-sha1 = "03a9b9718f5682ecb107ac9f7308991db4ce395b"
uuid = "739be429-bea8-5141-9913-cc70e7f3736d"
version = "1.1.7"

[[deps.MbedTLS_jll]]
deps = ["Artifacts", "Libdl"]
uuid = "c8ffd9c3-330d-5841-b78e-0817d7145fa1"
version = "2.28.0+0"

[[deps.Measures]]
git-tree-sha1 = "e498ddeee6f9fdb4551ce855a46f54dbd900245f"
uuid = "442fdcdd-2543-5da2-b0f3-8c86c306513e"
version = "0.3.1"

[[deps.Missings]]
deps = ["DataAPI"]
git-tree-sha1 = "bf210ce90b6c9eed32d25dbcae1ebc565df2687f"
uuid = "e1d29d7a-bbdc-5cf2-9ac0-f12de2c33e28"
version = "1.0.2"

[[deps.Mmap]]
uuid = "a63ad114-7e13-5084-954f-fe012c677804"

[[deps.MosaicViews]]
deps = ["MappedArrays", "OffsetArrays", "PaddedViews", "StackViews"]
git-tree-sha1 = "b34e3bc3ca7c94914418637cb10cc4d1d80d877d"
uuid = "e94cdb99-869f-56ef-bcf0-1ae2bcbe0389"
version = "0.3.3"

[[deps.MozillaCACerts_jll]]
uuid = "14a3606d-f60d-562e-9121-12d972cd8159"
version = "2022.2.1"

[[deps.NaNMath]]
deps = ["OpenLibm_jll"]
git-tree-sha1 = "a7c3d1da1189a1c2fe843a3bfa04d18d20eb3211"
uuid = "77ba4419-2d1f-58cd-9bb1-8ffee604a2e3"
version = "1.0.1"

[[deps.Netpbm]]
deps = ["FileIO", "ImageCore"]
git-tree-sha1 = "18efc06f6ec36a8b801b23f076e3c6ac7c3bf153"
uuid = "f09324ee-3d7c-5217-9330-fc30815ba969"
version = "1.0.2"

[[deps.NetworkOptions]]
uuid = "ca575930-c2e3-43a9-ace4-1e988b2c1908"
version = "1.2.0"

[[deps.OffsetArrays]]
deps = ["Adapt"]
git-tree-sha1 = "f71d8950b724e9ff6110fc948dff5a329f901d64"
uuid = "6fe1bfb0-de20-5000-8ca7-80f57d26f881"
version = "1.12.8"

[[deps.Ogg_jll]]
deps = ["Artifacts", "JLLWrappers", "Libdl", "Pkg"]
git-tree-sha1 = "887579a3eb005446d514ab7aeac5d1d027658b8f"
uuid = "e7412a2a-1a6e-54c0-be00-318e2571c051"
version = "1.3.5+1"

[[deps.OpenBLAS_jll]]
deps = ["Artifacts", "CompilerSupportLibraries_jll", "Libdl"]
uuid = "4536629a-c528-5b80-bd46-f80d51c5b363"
version = "0.3.20+0"

[[deps.OpenEXR]]
deps = ["Colors", "FileIO", "OpenEXR_jll"]
git-tree-sha1 = "327f53360fdb54df7ecd01e96ef1983536d1e633"
uuid = "52e1d378-f018-4a11-a4be-720524705ac7"
version = "0.3.2"

[[deps.OpenEXR_jll]]
deps = ["Artifacts", "Imath_jll", "JLLWrappers", "Libdl", "Pkg", "Zlib_jll"]
git-tree-sha1 = "923319661e9a22712f24596ce81c54fc0366f304"
uuid = "18a262bb-aa17-5467-a713-aee519bc75cb"
version = "3.1.1+0"

[[deps.OpenLibm_jll]]
deps = ["Artifacts", "Libdl"]
uuid = "05823500-19ac-5b8b-9628-191a04bc5112"
version = "0.8.1+0"

[[deps.OpenSSL]]
deps = ["BitFlags", "Dates", "MozillaCACerts_jll", "OpenSSL_jll", "Sockets"]
git-tree-sha1 = "3c3c4a401d267b04942545b1e964a20279587fd7"
uuid = "4d8831e6-92b7-49fb-bdf8-b643e874388c"
version = "1.3.0"

[[deps.OpenSSL_jll]]
deps = ["Artifacts", "JLLWrappers", "Libdl", "Pkg"]
git-tree-sha1 = "e60321e3f2616584ff98f0a4f18d98ae6f89bbb3"
uuid = "458c3c95-2e84-50aa-8efc-19380b2a3a95"
version = "1.1.17+0"

[[deps.OpenSpecFun_jll]]
deps = ["Artifacts", "CompilerSupportLibraries_jll", "JLLWrappers", "Libdl", "Pkg"]
git-tree-sha1 = "13652491f6856acfd2db29360e1bbcd4565d04f1"
uuid = "efe28fd5-8261-553b-a9e1-b2916fc3738e"
version = "0.5.5+0"

[[deps.Opus_jll]]
deps = ["Artifacts", "JLLWrappers", "Libdl", "Pkg"]
git-tree-sha1 = "51a08fb14ec28da2ec7a927c4337e4332c2a4720"
uuid = "91d4177d-7536-5919-b921-800302f37372"
version = "1.3.2+0"

[[deps.OrderedCollections]]
git-tree-sha1 = "85f8e6578bf1f9ee0d11e7bb1b1456435479d47c"
uuid = "bac558e1-5e72-5ebc-8fee-abe8a469f55d"
version = "1.4.1"

[[deps.PCRE2_jll]]
deps = ["Artifacts", "Libdl"]
uuid = "efcefdf7-47ab-520b-bdef-62a2eaa19f15"
version = "10.40.0+0"

[[deps.PNGFiles]]
deps = ["Base64", "CEnum", "ImageCore", "IndirectArrays", "OffsetArrays", "libpng_jll"]
git-tree-sha1 = "f809158b27eba0c18c269cf2a2be6ed751d3e81d"
uuid = "f57f5aa1-a3ce-4bc8-8ab9-96f992907883"
version = "0.3.17"

[[deps.PaddedViews]]
deps = ["OffsetArrays"]
git-tree-sha1 = "03a7a85b76381a3d04c7a1656039197e70eda03d"
uuid = "5432bcbf-9aad-5242-b902-cca2824c8663"
version = "0.5.11"

[[deps.Parsers]]
deps = ["Dates"]
git-tree-sha1 = "6c01a9b494f6d2a9fc180a08b182fcb06f0958a0"
uuid = "69de0a69-1ddd-5017-9359-2bf0b02dc9f0"
version = "2.4.2"

[[deps.Pipe]]
git-tree-sha1 = "6842804e7867b115ca9de748a0cf6b364523c16d"
uuid = "b98c9c47-44ae-5843-9183-064241ee97a0"
version = "1.3.0"

[[deps.Pixman_jll]]
deps = ["Artifacts", "JLLWrappers", "Libdl", "Pkg"]
git-tree-sha1 = "b4f5d02549a10e20780a24fce72bea96b6329e29"
uuid = "30392449-352a-5448-841d-b1acce4e97dc"
version = "0.40.1+0"

[[deps.Pkg]]
deps = ["Artifacts", "Dates", "Downloads", "LibGit2", "Libdl", "Logging", "Markdown", "Printf", "REPL", "Random", "SHA", "Serialization", "TOML", "Tar", "UUIDs", "p7zip_jll"]
uuid = "44cfe95a-1eb2-52ea-b672-e2afdf69b78f"
version = "1.8.0"

[[deps.PkgVersion]]
deps = ["Pkg"]
git-tree-sha1 = "f6cf8e7944e50901594838951729a1861e668cb8"
uuid = "eebad327-c553-4316-9ea0-9fa01ccd7688"
version = "0.3.2"

[[deps.PlotThemes]]
deps = ["PlotUtils", "Statistics"]
git-tree-sha1 = "1f03a2d339f42dca4a4da149c7e15e9b896ad899"
uuid = "ccf2f8ad-2431-5c83-bf29-c5338b663b6a"
version = "3.1.0"

[[deps.PlotUtils]]
deps = ["ColorSchemes", "Colors", "Dates", "Printf", "Random", "Reexport", "SnoopPrecompile", "Statistics"]
git-tree-sha1 = "21303256d239f6b484977314674aef4bb1fe4420"
uuid = "995b91a9-d308-5afd-9ec6-746e21dbc043"
version = "1.3.1"

[[deps.Plots]]
deps = ["Base64", "Contour", "Dates", "Downloads", "FFMPEG", "FixedPointNumbers", "GR", "JLFzf", "JSON", "LaTeXStrings", "Latexify", "LinearAlgebra", "Measures", "NaNMath", "Pkg", "PlotThemes", "PlotUtils", "Printf", "REPL", "Random", "RecipesBase", "RecipesPipeline", "Reexport", "RelocatableFolders", "Requires", "Scratch", "Showoff", "SnoopPrecompile", "SparseArrays", "Statistics", "StatsBase", "UUIDs", "UnicodeFun", "Unzip"]
git-tree-sha1 = "0a56829d264eb1bc910cf7c39ac008b5bcb5a0d9"
uuid = "91a5bcdd-55d7-5caf-9e0b-520d859cae80"
version = "1.35.5"

[[deps.PlutoUI]]
deps = ["AbstractPlutoDingetjes", "Base64", "ColorTypes", "Dates", "FixedPointNumbers", "Hyperscript", "HypertextLiteral", "IOCapture", "InteractiveUtils", "JSON", "Logging", "MIMEs", "Markdown", "Random", "Reexport", "URIs", "UUIDs"]
git-tree-sha1 = "efc140104e6d0ae3e7e30d56c98c4a927154d684"
uuid = "7f904dfe-b85e-4ff6-b463-dae2292396a8"
version = "0.7.48"

[[deps.Preferences]]
deps = ["TOML"]
git-tree-sha1 = "47e5f437cc0e7ef2ce8406ce1e7e24d44915f88d"
uuid = "21216c6a-2e73-6563-6e65-726566657250"
version = "1.3.0"

[[deps.Printf]]
deps = ["Unicode"]
uuid = "de0858da-6303-5e67-8744-51eddeeeb8d7"

[[deps.ProgressMeter]]
deps = ["Distributed", "Printf"]
git-tree-sha1 = "d7a7aef8f8f2d537104f170139553b14dfe39fe9"
uuid = "92933f4c-e287-5a05-a399-4b506db050ca"
version = "1.7.2"

[[deps.QOI]]
deps = ["ColorTypes", "FileIO", "FixedPointNumbers"]
git-tree-sha1 = "18e8f4d1426e965c7b532ddd260599e1510d26ce"
uuid = "4b34888f-f399-49d4-9bb3-47ed5cae4e65"
version = "1.0.0"

[[deps.Qt5Base_jll]]
deps = ["Artifacts", "CompilerSupportLibraries_jll", "Fontconfig_jll", "Glib_jll", "JLLWrappers", "Libdl", "Libglvnd_jll", "OpenSSL_jll", "Pkg", "Xorg_libXext_jll", "Xorg_libxcb_jll", "Xorg_xcb_util_image_jll", "Xorg_xcb_util_keysyms_jll", "Xorg_xcb_util_renderutil_jll", "Xorg_xcb_util_wm_jll", "Zlib_jll", "xkbcommon_jll"]
git-tree-sha1 = "c6c0f690d0cc7caddb74cef7aa847b824a16b256"
uuid = "ea2cea3b-5b76-57ae-a6ef-0a8af62496e1"
version = "5.15.3+1"

[[deps.REPL]]
deps = ["InteractiveUtils", "Markdown", "Sockets", "Unicode"]
uuid = "3fa0cd96-eef1-5676-8a61-b3b8758bbffb"

[[deps.Random]]
deps = ["SHA", "Serialization"]
uuid = "9a3f8284-a2c9-5f02-9a11-845980a1fd5c"

[[deps.RangeArrays]]
git-tree-sha1 = "b9039e93773ddcfc828f12aadf7115b4b4d225f5"
uuid = "b3c3ace0-ae52-54e7-9d0b-2c1406fd6b9d"
version = "0.3.2"

[[deps.RecipesBase]]
deps = ["SnoopPrecompile"]
git-tree-sha1 = "d12e612bba40d189cead6ff857ddb67bd2e6a387"
uuid = "3cdcf5f2-1ef4-517c-9805-6587b60abb01"
version = "1.3.1"

[[deps.RecipesPipeline]]
deps = ["Dates", "NaNMath", "PlotUtils", "RecipesBase", "SnoopPrecompile"]
git-tree-sha1 = "9b1c0c8e9188950e66fc28f40bfe0f8aac311fe0"
uuid = "01d81517-befc-4cb6-b9ec-a95719d0359c"
version = "0.6.7"

[[deps.Reexport]]
git-tree-sha1 = "45e428421666073eab6f2da5c9d310d99bb12f9b"
uuid = "189a3867-3050-52da-a836-e630ba90ab69"
version = "1.2.2"

[[deps.RelocatableFolders]]
deps = ["SHA", "Scratch"]
git-tree-sha1 = "90bc7a7c96410424509e4263e277e43250c05691"
uuid = "05181044-ff0b-4ac5-8273-598c1e38db00"
version = "1.0.0"

[[deps.Requires]]
deps = ["UUIDs"]
git-tree-sha1 = "838a3a4188e2ded87a4f9f184b4b0d78a1e91cb7"
uuid = "ae029012-a4dd-5104-9daa-d747884805df"
version = "1.3.0"

[[deps.SHA]]
uuid = "ea8e919c-243c-51af-8825-aaa63cd721ce"
version = "0.7.0"

[[deps.Scratch]]
deps = ["Dates"]
git-tree-sha1 = "f94f779c94e58bf9ea243e77a37e16d9de9126bd"
uuid = "6c6a2e73-6563-6170-7368-637461726353"
version = "1.1.1"

[[deps.Serialization]]
uuid = "9e88b42a-f829-5b0c-bbe9-9e923198166b"

[[deps.Showoff]]
deps = ["Dates", "Grisu"]
git-tree-sha1 = "91eddf657aca81df9ae6ceb20b959ae5653ad1de"
uuid = "992d4aef-0814-514b-bc4d-f2e9a6c4116f"
version = "1.0.3"

[[deps.SimpleBufferStream]]
git-tree-sha1 = "874e8867b33a00e784c8a7e4b60afe9e037b74e1"
uuid = "777ac1f9-54b0-4bf8-805c-2214025038e7"
version = "1.1.0"

[[deps.Sixel]]
deps = ["Dates", "FileIO", "ImageCore", "IndirectArrays", "OffsetArrays", "REPL", "libsixel_jll"]
git-tree-sha1 = "8fb59825be681d451c246a795117f317ecbcaa28"
uuid = "45858cf5-a6b0-47a3-bbea-62219f50df47"
version = "0.1.2"

[[deps.SnoopPrecompile]]
git-tree-sha1 = "f604441450a3c0569830946e5b33b78c928e1a85"
uuid = "66db9d55-30c0-4569-8b51-7e840670fc0c"
version = "1.0.1"

[[deps.Sockets]]
uuid = "6462fe0b-24de-5631-8697-dd941f90decc"

[[deps.SortingAlgorithms]]
deps = ["DataStructures"]
git-tree-sha1 = "b3363d7460f7d098ca0912c69b082f75625d7508"
uuid = "a2af1166-a08f-5f64-846c-94a0d3cef48c"
version = "1.0.1"

[[deps.SparseArrays]]
deps = ["LinearAlgebra", "Random"]
uuid = "2f01184e-e22b-5df5-ae63-d93ebab69eaf"

[[deps.SpecialFunctions]]
deps = ["ChainRulesCore", "IrrationalConstants", "LogExpFunctions", "OpenLibm_jll", "OpenSpecFun_jll"]
git-tree-sha1 = "d75bda01f8c31ebb72df80a46c88b25d1c79c56d"
uuid = "276daf66-3868-5448-9aa4-cd146d93841b"
version = "2.1.7"

[[deps.StackViews]]
deps = ["OffsetArrays"]
git-tree-sha1 = "46e589465204cd0c08b4bd97385e4fa79a0c770c"
uuid = "cae243ae-269e-4f55-b966-ac2d0dc13c15"
version = "0.1.1"

[[deps.Statistics]]
deps = ["LinearAlgebra", "SparseArrays"]
uuid = "10745b16-79ce-11e8-11f9-7d13ad32a3b2"

[[deps.StatsAPI]]
deps = ["LinearAlgebra"]
git-tree-sha1 = "f9af7f195fb13589dd2e2d57fdb401717d2eb1f6"
uuid = "82ae8749-77ed-4fe6-ae5f-f523153014b0"
version = "1.5.0"

[[deps.StatsBase]]
deps = ["DataAPI", "DataStructures", "LinearAlgebra", "LogExpFunctions", "Missings", "Printf", "Random", "SortingAlgorithms", "SparseArrays", "Statistics", "StatsAPI"]
git-tree-sha1 = "d1bf48bfcc554a3761a133fe3a9bb01488e06916"
uuid = "2913bbd2-ae8a-5f71-8c99-4fb6c76f3a91"
version = "0.33.21"

[[deps.StringDistances]]
deps = ["Distances", "StatsAPI"]
git-tree-sha1 = "ceeef74797d961aee825aabf71446d6aba898acb"
uuid = "88034a9c-02f8-509d-84a9-84ec65e18404"
version = "0.11.2"

[[deps.TOML]]
deps = ["Dates"]
uuid = "fa267f1f-6049-4f14-aa54-33bafae1ed76"
version = "1.0.0"

[[deps.Tar]]
deps = ["ArgTools", "SHA"]
uuid = "a4e569a6-e804-4fa4-b0f3-eef7a1d5b13e"
version = "1.10.0"

[[deps.TensorCore]]
deps = ["LinearAlgebra"]
git-tree-sha1 = "1feb45f88d133a655e001435632f019a9a1bcdb6"
uuid = "62fd8b95-f654-4bbd-a8a5-9c27f68ccd50"
version = "0.1.1"

[[deps.Test]]
deps = ["InteractiveUtils", "Logging", "Random", "Serialization"]
uuid = "8dfed614-e22c-5e08-85e1-65c5234f0b40"

[[deps.TestImages]]
deps = ["AxisArrays", "ColorTypes", "FileIO", "ImageIO", "ImageMagick", "OffsetArrays", "Pkg", "StringDistances"]
git-tree-sha1 = "03492434a1bdde3026288939fc31b5660407b624"
uuid = "5e47fb64-e119-507b-a336-dd2b206d9990"
version = "1.7.1"

[[deps.TiffImages]]
deps = ["ColorTypes", "DataStructures", "DocStringExtensions", "FileIO", "FixedPointNumbers", "IndirectArrays", "Inflate", "Mmap", "OffsetArrays", "PkgVersion", "ProgressMeter", "UUIDs"]
git-tree-sha1 = "70e6d2da9210371c927176cb7a56d41ef1260db7"
uuid = "731e570b-9d59-4bfa-96dc-6df516fadf69"
version = "0.6.1"

[[deps.TranscodingStreams]]
deps = ["Random", "Test"]
git-tree-sha1 = "8a75929dcd3c38611db2f8d08546decb514fcadf"
uuid = "3bb67fe8-82b1-5028-8e26-92a6c54297fa"
version = "0.9.9"

[[deps.Tricks]]
git-tree-sha1 = "6bac775f2d42a611cdfcd1fb217ee719630c4175"
uuid = "410a4b4d-49e4-4fbc-ab6d-cb71b17b3775"
version = "0.1.6"

[[deps.URIs]]
git-tree-sha1 = "e59ecc5a41b000fa94423a578d29290c7266fc10"
uuid = "5c2747f8-b7ea-4ff2-ba2e-563bfd36b1d4"
version = "1.4.0"

[[deps.UUIDs]]
deps = ["Random", "SHA"]
uuid = "cf7118a7-6976-5b1a-9a39-7adc72f591a4"

[[deps.Unicode]]
uuid = "4ec0a83e-493e-50e2-b9ac-8f72acf5a8f5"

[[deps.UnicodeFun]]
deps = ["REPL"]
git-tree-sha1 = "53915e50200959667e78a92a418594b428dffddf"
uuid = "1cfade01-22cf-5700-b092-accc4b62d6e1"
version = "0.4.1"

[[deps.Unzip]]
git-tree-sha1 = "ca0969166a028236229f63514992fc073799bb78"
uuid = "41fe7b60-77ed-43a1-b4f0-825fd5a5650d"
version = "0.2.0"

[[deps.Wayland_jll]]
deps = ["Artifacts", "Expat_jll", "JLLWrappers", "Libdl", "Libffi_jll", "Pkg", "XML2_jll"]
git-tree-sha1 = "3e61f0b86f90dacb0bc0e73a0c5a83f6a8636e23"
uuid = "a2964d1f-97da-50d4-b82a-358c7fce9d89"
version = "1.19.0+0"

[[deps.Wayland_protocols_jll]]
deps = ["Artifacts", "JLLWrappers", "Libdl", "Pkg"]
git-tree-sha1 = "4528479aa01ee1b3b4cd0e6faef0e04cf16466da"
uuid = "2381bf8a-dfd0-557d-9999-79630e7b1b91"
version = "1.25.0+0"

[[deps.XML2_jll]]
deps = ["Artifacts", "JLLWrappers", "Libdl", "Libiconv_jll", "Pkg", "Zlib_jll"]
git-tree-sha1 = "58443b63fb7e465a8a7210828c91c08b92132dff"
uuid = "02c8fc9c-b97f-50b9-bbe4-9be30ff0a78a"
version = "2.9.14+0"

[[deps.XSLT_jll]]
deps = ["Artifacts", "JLLWrappers", "Libdl", "Libgcrypt_jll", "Libgpg_error_jll", "Libiconv_jll", "Pkg", "XML2_jll", "Zlib_jll"]
git-tree-sha1 = "91844873c4085240b95e795f692c4cec4d805f8a"
uuid = "aed1982a-8fda-507f-9586-7b0439959a61"
version = "1.1.34+0"

[[deps.Xorg_libX11_jll]]
deps = ["Artifacts", "JLLWrappers", "Libdl", "Pkg", "Xorg_libxcb_jll", "Xorg_xtrans_jll"]
git-tree-sha1 = "5be649d550f3f4b95308bf0183b82e2582876527"
uuid = "4f6342f7-b3d2-589e-9d20-edeb45f2b2bc"
version = "1.6.9+4"

[[deps.Xorg_libXau_jll]]
deps = ["Artifacts", "JLLWrappers", "Libdl", "Pkg"]
git-tree-sha1 = "4e490d5c960c314f33885790ed410ff3a94ce67e"
uuid = "0c0b7dd1-d40b-584c-a123-a41640f87eec"
version = "1.0.9+4"

[[deps.Xorg_libXcursor_jll]]
deps = ["Artifacts", "JLLWrappers", "Libdl", "Pkg", "Xorg_libXfixes_jll", "Xorg_libXrender_jll"]
git-tree-sha1 = "12e0eb3bc634fa2080c1c37fccf56f7c22989afd"
uuid = "935fb764-8cf2-53bf-bb30-45bb1f8bf724"
version = "1.2.0+4"

[[deps.Xorg_libXdmcp_jll]]
deps = ["Artifacts", "JLLWrappers", "Libdl", "Pkg"]
git-tree-sha1 = "4fe47bd2247248125c428978740e18a681372dd4"
uuid = "a3789734-cfe1-5b06-b2d0-1dd0d9d62d05"
version = "1.1.3+4"

[[deps.Xorg_libXext_jll]]
deps = ["Artifacts", "JLLWrappers", "Libdl", "Pkg", "Xorg_libX11_jll"]
git-tree-sha1 = "b7c0aa8c376b31e4852b360222848637f481f8c3"
uuid = "1082639a-0dae-5f34-9b06-72781eeb8cb3"
version = "1.3.4+4"

[[deps.Xorg_libXfixes_jll]]
deps = ["Artifacts", "JLLWrappers", "Libdl", "Pkg", "Xorg_libX11_jll"]
git-tree-sha1 = "0e0dc7431e7a0587559f9294aeec269471c991a4"
uuid = "d091e8ba-531a-589c-9de9-94069b037ed8"
version = "5.0.3+4"

[[deps.Xorg_libXi_jll]]
deps = ["Artifacts", "JLLWrappers", "Libdl", "Pkg", "Xorg_libXext_jll", "Xorg_libXfixes_jll"]
git-tree-sha1 = "89b52bc2160aadc84d707093930ef0bffa641246"
uuid = "a51aa0fd-4e3c-5386-b890-e753decda492"
version = "1.7.10+4"

[[deps.Xorg_libXinerama_jll]]
deps = ["Artifacts", "JLLWrappers", "Libdl", "Pkg", "Xorg_libXext_jll"]
git-tree-sha1 = "26be8b1c342929259317d8b9f7b53bf2bb73b123"
uuid = "d1454406-59df-5ea1-beac-c340f2130bc3"
version = "1.1.4+4"

[[deps.Xorg_libXrandr_jll]]
deps = ["Artifacts", "JLLWrappers", "Libdl", "Pkg", "Xorg_libXext_jll", "Xorg_libXrender_jll"]
git-tree-sha1 = "34cea83cb726fb58f325887bf0612c6b3fb17631"
uuid = "ec84b674-ba8e-5d96-8ba1-2a689ba10484"
version = "1.5.2+4"

[[deps.Xorg_libXrender_jll]]
deps = ["Artifacts", "JLLWrappers", "Libdl", "Pkg", "Xorg_libX11_jll"]
git-tree-sha1 = "19560f30fd49f4d4efbe7002a1037f8c43d43b96"
uuid = "ea2f1a96-1ddc-540d-b46f-429655e07cfa"
version = "0.9.10+4"

[[deps.Xorg_libpthread_stubs_jll]]
deps = ["Artifacts", "JLLWrappers", "Libdl", "Pkg"]
git-tree-sha1 = "6783737e45d3c59a4a4c4091f5f88cdcf0908cbb"
uuid = "14d82f49-176c-5ed1-bb49-ad3f5cbd8c74"
version = "0.1.0+3"

[[deps.Xorg_libxcb_jll]]
deps = ["Artifacts", "JLLWrappers", "Libdl", "Pkg", "XSLT_jll", "Xorg_libXau_jll", "Xorg_libXdmcp_jll", "Xorg_libpthread_stubs_jll"]
git-tree-sha1 = "daf17f441228e7a3833846cd048892861cff16d6"
uuid = "c7cfdc94-dc32-55de-ac96-5a1b8d977c5b"
version = "1.13.0+3"

[[deps.Xorg_libxkbfile_jll]]
deps = ["Artifacts", "JLLWrappers", "Libdl", "Pkg", "Xorg_libX11_jll"]
git-tree-sha1 = "926af861744212db0eb001d9e40b5d16292080b2"
uuid = "cc61e674-0454-545c-8b26-ed2c68acab7a"
version = "1.1.0+4"

[[deps.Xorg_xcb_util_image_jll]]
deps = ["Artifacts", "JLLWrappers", "Libdl", "Pkg", "Xorg_xcb_util_jll"]
git-tree-sha1 = "0fab0a40349ba1cba2c1da699243396ff8e94b97"
uuid = "12413925-8142-5f55-bb0e-6d7ca50bb09b"
version = "0.4.0+1"

[[deps.Xorg_xcb_util_jll]]
deps = ["Artifacts", "JLLWrappers", "Libdl", "Pkg", "Xorg_libxcb_jll"]
git-tree-sha1 = "e7fd7b2881fa2eaa72717420894d3938177862d1"
uuid = "2def613f-5ad1-5310-b15b-b15d46f528f5"
version = "0.4.0+1"

[[deps.Xorg_xcb_util_keysyms_jll]]
deps = ["Artifacts", "JLLWrappers", "Libdl", "Pkg", "Xorg_xcb_util_jll"]
git-tree-sha1 = "d1151e2c45a544f32441a567d1690e701ec89b00"
uuid = "975044d2-76e6-5fbe-bf08-97ce7c6574c7"
version = "0.4.0+1"

[[deps.Xorg_xcb_util_renderutil_jll]]
deps = ["Artifacts", "JLLWrappers", "Libdl", "Pkg", "Xorg_xcb_util_jll"]
git-tree-sha1 = "dfd7a8f38d4613b6a575253b3174dd991ca6183e"
uuid = "0d47668e-0667-5a69-a72c-f761630bfb7e"
version = "0.3.9+1"

[[deps.Xorg_xcb_util_wm_jll]]
deps = ["Artifacts", "JLLWrappers", "Libdl", "Pkg", "Xorg_xcb_util_jll"]
git-tree-sha1 = "e78d10aab01a4a154142c5006ed44fd9e8e31b67"
uuid = "c22f9ab0-d5fe-5066-847c-f4bb1cd4e361"
version = "0.4.1+1"

[[deps.Xorg_xkbcomp_jll]]
deps = ["Artifacts", "JLLWrappers", "Libdl", "Pkg", "Xorg_libxkbfile_jll"]
git-tree-sha1 = "4bcbf660f6c2e714f87e960a171b119d06ee163b"
uuid = "35661453-b289-5fab-8a00-3d9160c6a3a4"
version = "1.4.2+4"

[[deps.Xorg_xkeyboard_config_jll]]
deps = ["Artifacts", "JLLWrappers", "Libdl", "Pkg", "Xorg_xkbcomp_jll"]
git-tree-sha1 = "5c8424f8a67c3f2209646d4425f3d415fee5931d"
uuid = "33bec58e-1273-512f-9401-5d533626f822"
version = "2.27.0+4"

[[deps.Xorg_xtrans_jll]]
deps = ["Artifacts", "JLLWrappers", "Libdl", "Pkg"]
git-tree-sha1 = "79c31e7844f6ecf779705fbc12146eb190b7d845"
uuid = "c5fb5394-a638-5e4d-96e5-b29de1b5cf10"
version = "1.4.0+3"

[[deps.Zlib_jll]]
deps = ["Libdl"]
uuid = "83775a58-1f1d-513f-b197-d71354ab007a"
version = "1.2.12+3"

[[deps.Zstd_jll]]
deps = ["Artifacts", "JLLWrappers", "Libdl", "Pkg"]
git-tree-sha1 = "e45044cd873ded54b6a5bac0eb5c971392cf1927"
uuid = "3161d3a3-bdf6-5164-811a-617609db77b4"
version = "1.5.2+0"

[[deps.fzf_jll]]
deps = ["Artifacts", "JLLWrappers", "Libdl", "Pkg"]
git-tree-sha1 = "868e669ccb12ba16eaf50cb2957ee2ff61261c56"
uuid = "214eeab7-80f7-51ab-84ad-2988db7cef09"
version = "0.29.0+0"

[[deps.libaom_jll]]
deps = ["Artifacts", "JLLWrappers", "Libdl", "Pkg"]
git-tree-sha1 = "3a2ea60308f0996d26f1e5354e10c24e9ef905d4"
uuid = "a4ae2306-e953-59d6-aa16-d00cac43593b"
version = "3.4.0+0"

[[deps.libass_jll]]
deps = ["Artifacts", "Bzip2_jll", "FreeType2_jll", "FriBidi_jll", "HarfBuzz_jll", "JLLWrappers", "Libdl", "Pkg", "Zlib_jll"]
git-tree-sha1 = "5982a94fcba20f02f42ace44b9894ee2b140fe47"
uuid = "0ac62f75-1d6f-5e53-bd7c-93b484bb37c0"
version = "0.15.1+0"

[[deps.libblastrampoline_jll]]
deps = ["Artifacts", "Libdl", "OpenBLAS_jll"]
uuid = "8e850b90-86db-534c-a0d3-1478176c7d93"
version = "5.1.1+0"

[[deps.libfdk_aac_jll]]
deps = ["Artifacts", "JLLWrappers", "Libdl", "Pkg"]
git-tree-sha1 = "daacc84a041563f965be61859a36e17c4e4fcd55"
uuid = "f638f0a6-7fb0-5443-88ba-1cc74229b280"
version = "2.0.2+0"

[[deps.libpng_jll]]
deps = ["Artifacts", "JLLWrappers", "Libdl", "Pkg", "Zlib_jll"]
git-tree-sha1 = "94d180a6d2b5e55e447e2d27a29ed04fe79eb30c"
uuid = "b53b4c65-9356-5827-b1ea-8c7a1a84506f"
version = "1.6.38+0"

[[deps.libsixel_jll]]
deps = ["Artifacts", "JLLWrappers", "JpegTurbo_jll", "Libdl", "Pkg", "libpng_jll"]
git-tree-sha1 = "d4f63314c8aa1e48cd22aa0c17ed76cd1ae48c3c"
uuid = "075b6546-f08a-558a-be8f-8157d0f608a5"
version = "1.10.3+0"

[[deps.libvorbis_jll]]
deps = ["Artifacts", "JLLWrappers", "Libdl", "Ogg_jll", "Pkg"]
git-tree-sha1 = "b910cb81ef3fe6e78bf6acee440bda86fd6ae00c"
uuid = "f27f6e37-5d2b-51aa-960f-b287f2bc3b7a"
version = "1.3.7+1"

[[deps.nghttp2_jll]]
deps = ["Artifacts", "Libdl"]
uuid = "8e850ede-7688-5339-a07c-302acd2aaf8d"
version = "1.48.0+0"

[[deps.p7zip_jll]]
deps = ["Artifacts", "Libdl"]
uuid = "3f19e933-33d8-53b3-aaab-bd5110c3b7a0"
version = "17.4.0+0"

[[deps.x264_jll]]
deps = ["Artifacts", "JLLWrappers", "Libdl", "Pkg"]
git-tree-sha1 = "4fea590b89e6ec504593146bf8b988b2c00922b2"
uuid = "1270edf5-f2f9-52d2-97e9-ab00b5d0237a"
version = "2021.5.5+0"

[[deps.x265_jll]]
deps = ["Artifacts", "JLLWrappers", "Libdl", "Pkg"]
git-tree-sha1 = "ee567a171cce03570d77ad3a43e90218e38937a9"
uuid = "dfaa095f-4041-5dcd-9319-2fabd8486b76"
version = "3.5.0+0"

[[deps.xkbcommon_jll]]
deps = ["Artifacts", "JLLWrappers", "Libdl", "Pkg", "Wayland_jll", "Wayland_protocols_jll", "Xorg_libxcb_jll", "Xorg_xkeyboard_config_jll"]
git-tree-sha1 = "9ebfc140cc56e8c2156a15ceac2f0302e327ac0a"
uuid = "d8fb68d0-12a3-5cfd-a85a-d49703b185fd"
version = "1.4.1+0"
"""

# ╔═╡ Cell order:
# ╟─349d7534-6212-11eb-2bc5-db5be39b6bb6
# ╟─369a24c8-6f09-11eb-16a9-9f2c6c94a77f
# ╟─fcb548d6-672a-11eb-3167-e7da4dfc9126
# ╟─6ca05a0a-915b-11eb-07d2-a7277ce215d6
# ╟─edd72f7e-9159-11eb-220a-77decc54c26b
# ╟─44cd3c42-915a-11eb-060f-f705832f2ec2
# ╟─679ce7ce-9176-11eb-0522-8fcbd56eb606
# ╟─581a8266-672f-11eb-0945-ff78763e7ffe
# ╠═6d646d66-672e-11eb-3380-45208037b0fd
# ╟─9849f642-672f-11eb-2ccf-db35a28f9623
# ╠═1df4f1f4-6730-11eb-3577-cfce50db2306
# ╟─238263a4-6730-11eb-3f01-c727f2a2653f
# ╠═a26d2fb6-6730-11eb-0335-5903a9c7e1e3
# ╠═b2b38c2e-6730-11eb-16cb-dbf9b0bc77c6
# ╠═c74fbb44-6730-11eb-2d65-d399710e4731
# ╠═cecc9b94-6730-11eb-0fbe-af224c6d9333
# ╟─dae44ee0-6730-11eb-3d9e-070805ec03f8
# ╠═15d5224c-6731-11eb-3cac-7965f2219d70
# ╟─4aa92996-6731-11eb-2f37-cf4c344cfe36
# ╠═2c28625a-6731-11eb-2a7b-fb9a8cc66c4c
# ╠═41cbcdae-6731-11eb-0744-e37ed9e38bd6
# ╟─8b1cf122-6731-11eb-3cc3-313aa4eeeaad
# ╟─99738086-9140-11eb-3976-871ba51b4001
# ╟─983ad5ba-9141-11eb-189f-49719162960f
# ╟─e99b310c-9141-11eb-00dd-2b12f8ff306d
# ╠═aac7e61a-9140-11eb-3ef9-479016cc327f
# ╟─6a905992-9142-11eb-0661-e934fb259bcd
# ╟─68d9602e-6730-11eb-3749-c326974bbc32
# ╠═ef5b41b0-90cf-11eb-10d6-c77a24888854
# ╠═0ec6d94e-90d0-11eb-280f-c72dae3ba87a
# ╟─149d66a8-90d0-11eb-0d06-4d0136e49046
# ╟─ef167287-8a5c-42b5-a201-53a01f49d771
# ╟─55589bec-90d1-11eb-2f86-45ecfcd979aa
# ╠═8520088a-90d1-11eb-1315-d18ad7cb7272
# ╠═f12348cc-90d2-11eb-3a36-474cb43c61e5
# ╟─0a6f00da-90d3-11eb-0809-17e905c457bb
# ╠═866f0c20-90d3-11eb-3bf4-c1d1b8fcf7d0
# ╟─92158d40-90d0-11eb-1a22-7dcb06bad9c0
# ╠═7f150096-90d4-11eb-12ff-b30b4678f278
# ╟─b79a024a-90d4-11eb-3dd5-e9a39c72094b
# ╟─9fbf1526-6f09-11eb-1fa1-bd7168b987af
# ╠═5ec086c6-90d9-11eb-0606-6d7c3252d55b
# ╟─7ad77c7e-90da-11eb-2082-2ff8ffb611d0
# ╠═b02fe60e-90da-11eb-2808-7f013ef47d5e
# ╟─dd18ebc0-90da-11eb-3969-63f78fffa36a
# ╟─2c393392-673d-4f8d-a658-0572817eb795
# ╟─851a5dd6-2580-412a-8ba6-ba724c56d265
# ╟─99d34166-6f09-11eb-0c40-855d2ea50483
# ╟─3ab650b8-66c1-4b1a-8e02-ea4f122b1f90
# ╟─a5a4b622-8b85-4d3b-b469-79d125d5b0b2
# ╠═334b9391-74e8-4778-bd71-eebc54d90fde
# ╟─b58c1f71-11b7-4dc5-8ebb-69f9e983f7a3
# ╠═7b5abd7f-68de-43fc-bd6f-8ecc8a62a8c5
# ╟─ba247887-6163-4052-a075-f0ea93205f3e
# ╟─c5243da6-e25c-43ba-ae4a-be8ee3f3e67c
# ╠═c79bbdd6-20e2-4669-a3fd-ef6632125b7b
# ╟─7d0300b9-bd0d-40ec-ba73-f2c2150b129e
# ╟─7e0cc06a-e506-4679-a063-dd9396e5370e
# ╟─f6fcf5b4-90ca-11eb-0650-87d8b0fef88b
# ╠═a5d02002-90cb-11eb-2e9b-ef925998dd8b
# ╠═dddb1c0e-90cb-11eb-1fdd-c97b40c87ef9
# ╟─240744fa-90cc-11eb-3193-55bd4f2dbdcc
# ╠═cf2214aa-90cc-11eb-30b7-d350c3d6ba58
# ╠═e99bba02-90cc-11eb-1673-879c2f2fd4ef
# ╟─f95dfd42-90cc-11eb-2fa8-07335bfa0588
# ╟─a0bbd8bf-d3c0-4509-8fae-77c7ad65d70e
# ╠═ae3aa6a3-8c30-4f54-a84f-ba9edbc4c748
# ╟─362ee388-0bf9-4100-9ca1-2b527df8a96c
# ╠═8e29da94-b262-4146-aa76-6d23723d6482
# ╟─c31c19f8-d2a1-47de-9331-55600842fc99
# ╠═b4bc673d-d8c8-4e09-afa6-a4931c9b41d4
# ╟─d8012453-0aae-40c2-a20d-d37b100795d2
# ╟─c2b497d0-913d-11eb-05d6-7955d6ec6c74
# ╠═58cfa7d2-913e-11eb-05d6-ff28c8c8498d
# ╟─74d68aae-913e-11eb-232f-13ad436686ae
# ╠═ea7bdc18-913f-11eb-0272-f1a5ad0e721c
# ╟─bb557874-9147-11eb-1668-39a5cba21dc1
# ╠═d504529a-9147-11eb-38ad-2f40c72edff4
# ╟─37092042-9148-11eb-338b-0d34614df7fb
# ╟─5841b1a2-9148-11eb-08ab-4d3961a05288
# ╠═ca152ffc-9157-11eb-08f2-ad9a68f6b6d9
# ╟─6d157b32-9159-11eb-0af7-d5065c5f93a6
# ╠═87b344da-9159-11eb-206b-afa666055114
# ╟─c3e653c8-9147-11eb-0df1-39b2adca1a2f
# ╟─a693f6d8-9148-11eb-1ad1-a151deaac3f3
# ╠═02e65534-9178-11eb-191e-41a55663e13b
# ╟─ad371550-9178-11eb-3163-278104dd8f22
# ╠═d15c5d0a-9178-11eb-0bdd-d7fac5214a12
# ╟─6c14d16a-917b-11eb-2a88-6bb1acc7b54d
# ╟─2f73f10a-917c-11eb-2ab7-f52a22898d60
# ╠═4680264a-917c-11eb-38d9-69a3b5edbe0c
# ╟─5b72463a-9177-11eb-002d-8fccb44d6cce
# ╠═a2044a4e-9179-11eb-24c7-b18c52aa2201
# ╟─a2df0952-9179-11eb-33b0-470431612d5b
# ╠═965d94b0-917a-11eb-126c-e38b079e5225
# ╟─0877c6ba-917b-11eb-15da-2f65cd32c8b7
# ╟─b233611a-9148-11eb-3e0f-29871afbf62e
# ╠═bbb5798a-9148-11eb-0f24-95a8f57181b0
# ╟─3fa7226e-299c-4b60-b78e-4f22839a3418
# ╠═34ae7ce1-495f-4c89-b5f6-5e5ee9a5b8a2
# ╟─67808046-cfd8-4c0e-8412-50a8f869dd6c
# ╠═47b6cf42-28ae-457d-af21-0bfc6fe58504
# ╟─17cc99a1-812f-40de-a057-8d913662d160
# ╟─722695de-6735-11eb-1fa2-f3acb00898dc
# ╠═dfb9cb84-6735-11eb-28e7-6112c7c78cdf
# ╟─0030fa2c-6736-11eb-354b-05a6d2a58fd5
# ╠═10a19182-6736-11eb-0034-71e41dc49f5f
# ╟─208dcb92-6736-11eb-1075-ef4230aafe3e
# ╠═370b5416-6736-11eb-2b6e-c3a96ab2979f
# ╠═43909a02-6736-11eb-09c0-dd1b8d80d0a4
# ╟─603d715e-6736-11eb-324e-8d5698fb7200
# ╠═8f4b9abe-6736-11eb-21cf-1564d4ab6047
# ╟─9b0585b8-6736-11eb-386e-83fae463664b
# ╟─e7c7ef6c-6736-11eb-02cd-0d990b0aea52
# ╠═35857026-6737-11eb-0d84-35926fcba6e6
# ╠═56934680-6737-11eb-38df-09c036804ad7
# ╠═6d489466-6737-11eb-281d-7dd70b29b50e
# ╟─7f33c86c-6737-11eb-0552-19ff450139b3
# ╠═423711c0-6f06-11eb-3440-b5eb20e3305c
# ╟─2a20cf98-9160-11eb-2e36-2963f159811a
# ╟─f5daef7a-9164-11eb-1dba-01580661c18a
# ╟─3521ac82-9165-11eb-29b7-bdd187f0264a
# ╟─4d8c448e-9166-11eb-1741-3d968ac8f36d
# ╟─ad734b64-9165-11eb-28b7-b5f91838c755
# ╟─2c8700c0-9135-11eb-11b7-3163d4665332
# ╠═d7730092-9167-11eb-2091-8d6f0f3c8e22
# ╟─00000000-0000-0000-0000-000000000001
# ╟─00000000-0000-0000-0000-000000000002
